@@ -1,131 +1,62 @@
 /***************************************************************************
  * ncat_connect.c -- Ncat connect mode.                                    *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
- *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2019 Insecure.Com LLC ("The Nmap  *
- * Project"). Nmap is also a registered trademark of the Nmap Project.     *
- * This program is free software; you may redistribute and/or modify it    *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE   *
- * CLARIFICATIONS AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your   *
- * right to use, modify, and redistribute this software under certain      *
- * conditions.  If you wish to embed Nmap technology into proprietary      *
- * software, we sell alternative licenses (contact sales@nmap.com).        *
- * Dozens of software vendors already license Nmap technology such as      *
- * host discovery, port scanning, OS detection, version detection, and     *
- * the Nmap Scripting Engine.                                              *
- *                                                                         *
- * Note that the GPL places important restrictions on "derivative works",  *
- * yet it does not provide a detailed definition of that term.  To avoid   *
- * misunderstandings, we interpret that term as broadly as copyright law   *
- * allows.  For example, we consider an application to constitute a        *
- * derivative work for the purpose of this license if it does any of the   *
- * following with any software or content covered by this license          *
- * ("Covered Software"):                                                   *
- *                                                                         *
- * o Integrates source code from Covered Software.                         *
- *                                                                         *
- * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
- * or nmap-service-probes.                                                 *
- *                                                                         *
- * o Is designed specifically to execute Covered Software and parse the    *
- * results (as opposed to typical shell or execution-menu apps, which will *
- * execute anything you tell them to).                                     *
- *                                                                         *
- * o Includes Covered Software in a proprietary executable installer.  The *
- * installers produced by InstallShield are an example of this.  Including *
- * Nmap with other software in compressed or archival form does not        *
- * trigger this provision, provided appropriate open source decompression  *
- * or de-archiving software is widely available for no charge.  For the    *
- * purposes of this license, an installer is considered to include Covered *
- * Software even if it actually retrieves a copy of Covered Software from  *
- * another source during runtime (such as by downloading it from the       *
- * Internet).                                                              *
- *                                                                         *
- * o Links (statically or dynamically) to a library which does any of the  *
- * above.                                                                  *
- *                                                                         *
- * o Executes a helper program, module, or script to do any of the above.  *
- *                                                                         *
- * This list is not exclusive, but is meant to clarify our interpretation  *
- * of derived works with some common examples.  Other people may interpret *
- * the plain GPL differently, so we consider this a special exception to   *
- * the GPL that we apply to Covered Software.  Works which meet any of     *
- * these conditions must conform to all of the terms of this license,      *
- * particularly including the GPL Section 3 requirements of providing      *
- * source code and allowing free redistribution of the work as a whole.    *
- *                                                                         *
- * As another special exception to the GPL terms, the Nmap Project grants  *
- * permission to link the code of this program with any version of the     *
- * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
- * linked combinations including the two.                                  *
- *                                                                         *
- * The Nmap Project has permission to redistribute Npcap, a packet         *
- * capturing driver and library for the Microsoft Windows platform.        *
- * Npcap is a separate work with it's own license rather than this Nmap    *
- * license.  Since the Npcap license does not permit redistribution        *
- * without special permission, our Nmap Windows binary packages which      *
- * contain Npcap may not be redistributed without special permission.      *
- *                                                                         *
- * Any redistribution of Covered Software, including any derived works,    *
- * must obey and carry forward all of the terms of this license, including *
- * obeying all GPL rules and restrictions.  For example, source code of    *
- * the whole work must be provided and free redistribution must be         *
- * allowed.  All GPL references to "this License", are to be treated as    *
- * including the terms and conditions of this license text as well.        *
- *                                                                         *
- * Because this license imposes special exceptions to the GPL, Covered     *
- * Work may not be combined (even as part of a larger work) with plain GPL *
- * software.  The terms, conditions, and exceptions of this license must   *
- * be included as well.  This license is incompatible with some other open *
- * source licenses as well.  In some cases we can relicense portions of    *
- * Nmap or grant special permissions to use it in other open source        *
- * software.  Please contact fyodor@nmap.org with any such requests.       *
- * Similarly, we don't incorporate incompatible open source software into  *
- * Covered Software without special permission from the copyright holders. *
- *                                                                         *
- * If you have any questions about the licensing restrictions on using     *
- * Nmap in other works, we are happy to help.  As mentioned above, we also *
- * offer an alternative license to integrate Nmap into proprietary         *
- * applications and appliances.  These contracts have been sold to dozens  *
- * of software vendors, and generally include a perpetual license as well  *
- * as providing support and updates.  They also fund the continued         *
- * development of Nmap.  Please email sales@nmap.com for further           *
- * information.                                                            *
- *                                                                         *
- * If you have received a written license agreement or contract for        *
- * Covered Software stating terms other than these, you may choose to use  *
- * and redistribute Covered Software under those terms instead of these.   *
- *                                                                         *
- * Source is provided to this software because we believe users have a     *
- * right to know exactly what a program is going to do before they run it. *
- * This also allows you to audit the software for security holes.          *
- *                                                                         *
- * Source code also allows you to port Nmap to new platforms, fix bugs,    *
- * and add new features.  You are highly encouraged to send your changes   *
- * to the dev@nmap.org mailing list for possible incorporation into the    *
- * main distribution.  By sending these changes to Fyodor or one of the    *
- * Insecure.Org development mailing lists, or checking them into the Nmap  *
- * source code repository, it is understood (unless you specify            *
- * otherwise) that you are offering the Nmap Project the unlimited,        *
- * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because     *
- * the inability to relicense code has caused devastating problems for     *
- * other Free Software projects (such as KDE and NASM).  We also           *
- * occasionally relicense the code to third parties as discussed above.    *
- * If you wish to specify special license conditions of your               *
- * contributions, just say so when you send them.                          *
- *                                                                         *
- * This program is distributed in the hope that it will be useful, but     *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
- * license file for more details (it's in a COPYING file included with     *
- * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
- *                                                                         *
+ *
+ * The Nmap Security Scanner is (C) 1996-2024 Nmap Software LLC ("The Nmap
+ * Project"). Nmap is also a registered trademark of the Nmap Project.
+ *
+ * This program is distributed under the terms of the Nmap Public Source
+ * License (NPSL). The exact license text applying to a particular Nmap
+ * release or source code control revision is contained in the LICENSE
+ * file distributed with that version of Nmap or source code control
+ * revision. More Nmap copyright/legal information is available from
+ * https://nmap.org/book/man-legal.html, and further information on the
+ * NPSL license itself can be found at https://nmap.org/npsl/ . This
+ * header summarizes some key points from the Nmap license, but is no
+ * substitute for the actual license text.
+ *
+ * Nmap is generally free for end users to download and use themselves,
+ * including commercial use. It is available from https://nmap.org.
+ *
+ * The Nmap license generally prohibits companies from using and
+ * redistributing Nmap in commercial products, but we sell a special Nmap
+ * OEM Edition with a more permissive license and special features for
+ * this purpose. See https://nmap.org/oem/
+ *
+ * If you have received a written Nmap license agreement or contract
+ * stating terms other than these (such as an Nmap OEM license), you may
+ * choose to use and redistribute Nmap under those terms instead.
+ *
+ * The official Nmap Windows builds include the Npcap software
+ * (https://npcap.com) for packet capture and transmission. It is under
+ * separate license terms which forbid redistribution without special
+ * permission. So the official Nmap Windows builds may not be redistributed
+ * without special permission (such as an Nmap OEM license).
+ *
+ * Source is provided to this software because we believe users have a
+ * right to know exactly what a program is going to do before they run it.
+ * This also allows you to audit the software for security holes.
+ *
+ * Source code also allows you to port Nmap to new platforms, fix bugs, and
+ * add new features. You are highly encouraged to submit your changes as a
+ * Github PR or by email to the dev@nmap.org mailing list for possible
+ * incorporation into the main distribution. Unless you specify otherwise, it
+ * is understood that you are offering us very broad rights to use your
+ * submissions as described in the Nmap Public Source License Contributor
+ * Agreement. This is important because we fund the project by selling licenses
+ * with various terms, and also because the inability to relicense code has
+ * caused devastating problems for other Free Software projects (such as KDE
+ * and NASM).
+ *
+ * The free version of Nmap is distributed in the hope that it will be
+ * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,
+ * indemnification and commercial support are all available through the
+ * Npcap OEM program--see https://nmap.org/oem/
+ *
  ***************************************************************************/
 
-/* $Id: ncat_connect.c 37640 2019-05-28 21:36:04Z dmiller $ */
+/* $Id: ncat_connect.c 38829 2024-04-02 21:32:24Z dmiller $ */
 
 #include "base64.h"
 #include "nsock.h"
@@ -148,6 +79,11 @@
 #ifdef HAVE_OPENSSL
 #include <openssl/ssl.h>
 #include <openssl/err.h>
+
+/* Deprecated in OpenSSL 3.0 */
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
+# define SSL_get_peer_certificate SSL_get1_peer_certificate
+#endif
 #endif
 
 #ifdef WIN32
@@ -258,8 +194,6 @@ static void set_ssl_ctx_options(SSL_CTX *ctx)
         bye("Unable to set OpenSSL cipher list: %s", ERR_error_string(ERR_get_error(), NULL));
     }
 
-#ifdef HAVE_ALPN_SUPPORT
-
     if (o.sslalpn) {
         size_t alpn_len;
         unsigned char *alpn = next_protos_parse(&alpn_len, o.sslalpn);
@@ -279,8 +213,6 @@ static void set_ssl_ctx_options(SSL_CTX *ctx)
         free(alpn);
     }
 
-#endif
-
 }
 #endif
 
@@ -293,6 +225,13 @@ static void connect_report(nsock_iod nsi)
     nsock_iod_get_communication_info(nsi, NULL, NULL, NULL, &peer.sockaddr,
                                      sizeof(peer.storage));
     if (o.verbose) {
+        char peer_str[INET6_ADDRSTRLEN + sizeof(union sockaddr_u)] = {0};
+        if (o.proxytype) {
+            Snprintf(peer_str, sizeof(peer_str), "%s:%u", o.target, o.portno);
+        }
+        else {
+            Strncpy(peer_str, socktop(&peer, 0), sizeof(peer_str));
+        }
 #ifdef HAVE_OPENSSL
         if (nsock_iod_check_ssl(nsi)) {
             X509 *cert;
@@ -300,8 +239,7 @@ static void connect_report(nsock_iod nsi)
             char digest_buf[SHA1_STRING_LENGTH + 1];
             char *fp;
 
-            loguser("SSL connection to %s:%d.", inet_socktop(&peer),
-                    nsock_iod_get_peerport(nsi));
+            loguser("SSL connection to %s.", peer_str);
 
             cert = SSL_get_peer_certificate((SSL *)nsock_iod_get_ssl(nsi));
             ncat_assert(cert != NULL);
@@ -321,24 +259,11 @@ static void connect_report(nsock_iod nsi)
             fp = ssl_cert_fp_str_sha1(cert, digest_buf, sizeof(digest_buf));
             ncat_assert(fp == digest_buf);
             loguser("SHA-1 fingerprint: %s\n", digest_buf);
-        } else {
-#if HAVE_SYS_UN_H
-            if (peer.sockaddr.sa_family == AF_UNIX)
-                loguser("Connected to %s.\n", peer.un.sun_path);
-            else
+        } else
 #endif
-                loguser("Connected to %s:%d.\n", inet_socktop(&peer),
-                        nsock_iod_get_peerport(nsi));
+        {
+            loguser("Connected to %s.\n", peer_str);
         }
-#else
-#if HAVE_SYS_UN_H
-        if (peer.sockaddr.sa_family == AF_UNIX)
-            loguser("Connected to %s.\n", peer.un.sun_path);
-        else
-#endif
-            loguser("Connected to %s:%d.\n", inet_socktop(&peer),
-                    nsock_iod_get_peerport(nsi));
-#endif
     }
 }
 
@@ -441,7 +366,6 @@ static int do_proxy_http(void)
     char *target;
     union sockaddr_u addr;
     size_t sslen;
-    void *addrbuf;
     char addrstr[INET6_ADDRSTRLEN];
 
     request = NULL;
@@ -465,17 +389,7 @@ static int do_proxy_http(void)
         target = o.target;
     } else {
         /* addr is now populated with either sockaddr_in or sockaddr_in6 */
-        switch (addr.sockaddr.sa_family) {
-            case AF_INET:
-                addrbuf = &addr.in.sin_addr;
-                break;
-            case AF_INET6:
-                addrbuf = &addr.in6.sin6_addr;
-                break;
-            default:
-                ncat_assert(0);
-        }
-        inet_ntop(addr.sockaddr.sa_family, addrbuf, addrstr, sizeof(addrstr));
+        Strncpy(addrstr, inet_socktop(&addr), sizeof(addrstr));
         target = addrstr;
         if (o.verbose && getaddrfamily(o.target) == -1)
             loguser("Host %s locally resolved to %s.\n", o.target, target);
@@ -600,7 +514,6 @@ bail:
  */
 static int do_proxy_socks4(void)
 {
-    struct socket_buffer stateful_buf;
     char socksbuf[8];
     struct socks4_data socks4msg;
     size_t datalen;
@@ -619,7 +532,6 @@ static int do_proxy_socks4(void)
         loguser("Proxy connection failed: %s.\n", socket_strerror(socket_errno()));
         return sd;
     }
-    socket_buffer_init(&stateful_buf, sd);
 
     if (o.verbose) {
         loguser("Connected to proxy %s:%hu\n", inet_socktop(&targetaddrs->addr),
@@ -662,7 +574,7 @@ static int do_proxy_socks4(void)
         socks4msg.address = addr.in.sin_addr.s_addr;
         if (o.verbose && getaddrfamily(o.target) == -1)
             loguser("Host %s locally resolved to %s.\n", o.target,
-                inet_ntoa(addr.in.sin_addr));
+                inet_socktop(&addr));
     }
 
     if (send(sd, (char *)&socks4msg, offsetof(struct socks4_data, data) + datalen, 0) < 0) {
@@ -673,7 +585,7 @@ static int do_proxy_socks4(void)
 
     /* The size of the socks4 response is 8 bytes. So read exactly
        8 bytes from the buffer */
-    if (socket_buffer_readcount(&stateful_buf, socksbuf, 8) < 0) {
+    if (recv(sd, socksbuf, 8, 0) < 0) {
         loguser("Error: short response from proxy.\n");
         close(sd);
         return -1;
@@ -694,10 +606,9 @@ static int do_proxy_socks4(void)
  */
 static int do_proxy_socks5(void)
 {
-    struct socket_buffer stateful_buf;
     struct socks5_connect socks5msg;
     uint16_t proxyport = htons(o.portno);
-    char socksbuf[8];
+    char socksbuf[4];
     int sd;
     size_t dstlen, targetlen;
     struct socks5_request socks5msg2;
@@ -708,7 +619,8 @@ static int do_proxy_socks5(void)
     size_t sslen;
     void *addrbuf;
     size_t addrlen;
-    char addrstr[INET6_ADDRSTRLEN];
+    size_t bndaddrlen;
+    char bndaddr[SOCKS5_DST_MAXLEN + 2]; /* IPv4/IPv6/hostname and port */
 
     sd = do_connect(SOCK_STREAM);
     if (sd == -1) {
@@ -716,8 +628,6 @@ static int do_proxy_socks5(void)
         return sd;
     }
 
-    socket_buffer_init(&stateful_buf, sd);
-
     if (o.verbose) {
         loguser("Connected to proxy %s:%hu\n", inet_socktop(&targetaddrs->addr),
             inet_port(&targetaddrs->addr));
@@ -737,15 +647,15 @@ static int do_proxy_socks5(void)
         return -1;
     }
 
-    /* first response just two bytes, version and auth method */
-    if (socket_buffer_readcount(&stateful_buf, socksbuf, 2) < 0) {
-        loguser("Error: malformed first response from proxy.\n");
+    /* connect response just two bytes, version and auth method */
+    if (recv(sd, socksbuf, 2, 0) < 0) {
+        loguser("Error: malformed connect response from proxy.\n");
         close(sd);
         return -1;
     }
 
     if (socksbuf[0] != SOCKS5_VERSION) {
-        loguser("Error: got wrong server version in response.\n");
+        loguser("Error: wrong SOCKS version in connect response.\n");
         close(sd);
         return -1;
     }
@@ -800,8 +710,8 @@ static int do_proxy_socks5(void)
              * Server response for username/password authentication:
              * field 1: version, 1 byte
              * field 2: status code, 1 byte.
-             * 0x00 = success
-             * any other value = failure, connection must be closed
+             *          0x00 = success
+             *          any other value = failure, connection must be closed
              */
 
             socks5auth.ver = 1;
@@ -820,7 +730,7 @@ static int do_proxy_socks5(void)
                 return -1;
             }
 
-            if (socket_buffer_readcount(&stateful_buf, socksbuf, 2) < 0) {
+            if (recv(sd, socksbuf, 2, 0) < 0) {
                 loguser("Error: malformed proxy authentication response.\n");
                 close(sd);
                 return -1;
@@ -861,7 +771,7 @@ static int do_proxy_socks5(void)
         if (o.verbose)
             loguser("Host %s will be resolved by the proxy.\n", o.target);
         socks5msg2.atyp = SOCKS5_ATYP_NAME;
-        targetlen=strlen(o.target);
+        targetlen = strlen(o.target);
         if (targetlen > SOCKS5_DST_MAXLEN){
             loguser("Error: hostname length exceeds %d.\n", SOCKS5_DST_MAXLEN);
             close(sd);
@@ -891,7 +801,7 @@ static int do_proxy_socks5(void)
         dstlen = addrlen;
         if (o.verbose && getaddrfamily(o.target) == -1)
             loguser("Host %s locally resolved to %s.\n", o.target,
-                inet_ntop(addr.sockaddr.sa_family, addrbuf, addrstr, sizeof(addrstr)));
+                inet_socktop(&addr));
     }
 
     memcpy(socks5msg2.dst + dstlen, &proxyport, 2);
@@ -903,9 +813,14 @@ static int do_proxy_socks5(void)
         return -1;
     }
 
-    /* TODO just two bytes for now, need to read more for bind */
-    if (socket_buffer_readcount(&stateful_buf, socksbuf, 2) < 0) {
-        loguser("Error: malformed second response from proxy.\n");
+    if (recv(sd, socksbuf, 4, 0) < 0) {
+        loguser("Error: malformed request response from proxy.\n");
+        close(sd);
+        return -1;
+    }
+
+    if (socksbuf[0] != SOCKS5_VERSION) {
+        loguser("Error: wrong SOCKS version in request response.\n");
         close(sd);
         return -1;
     }
@@ -953,6 +868,33 @@ static int do_proxy_socks5(void)
             return -1;
     }
 
+    switch (socksbuf[3]) {
+    case SOCKS5_ATYP_IPv4:
+        bndaddrlen = 4 + 2;
+        break;
+    case SOCKS5_ATYP_IPv6:
+        bndaddrlen = 16 + 2;
+        break;
+    case SOCKS5_ATYP_NAME:
+        if (recv(sd, socksbuf, 1, 0) < 0) {
+            loguser("Error: malformed request response from proxy.\n");
+            close(sd);
+            return -1;
+        }
+        bndaddrlen = (unsigned char)socksbuf[0] + 2;
+        break;
+    default:
+        loguser("Error: invalid proxy bind address type.\n");
+        close(sd);
+        return -1;
+    }
+
+    if (recv(sd, bndaddr, bndaddrlen, 0) < 0) {
+        loguser("Error: malformed request response from proxy.\n");
+        close(sd);
+        return -1;
+    }
+
     return(sd);
 }
 
@@ -960,7 +902,7 @@ static nsock_iod new_iod(nsock_pool mypool) {
    nsock_iod nsi = nsock_iod_new(mypool, NULL);
    if (nsi == NULL)
      bye("Failed to create nsock_iod.");
-   if (nsock_iod_set_hostname(nsi, o.target) == -1)
+   if (nsock_iod_set_hostname(nsi, o.sslservername) == -1)
      bye("Failed to set hostname on iod.");
 
    switch (srcaddr.storage.ss_family) {
@@ -1031,7 +973,7 @@ int ncat_connect(void)
     nsock_pool_set_broadcast(mypool, 1);
 
 #ifdef HAVE_OPENSSL
-#ifdef HAVE_DTLS_CLIENT_METHOD
+#ifndef OPENSSL_NO_DTLS
     if(o.proto == IPPROTO_UDP)
         set_ssl_ctx_options((SSL_CTX *) nsock_pool_dtls_init(mypool, 0));
     else
@@ -1061,8 +1003,7 @@ int ncat_connect(void)
                     bye("Failed to create name for temporary DGRAM source Unix domain socket (tempnam).");
 #endif
 
-                srcaddr.un.sun_family = AF_UNIX;
-                strncpy(srcaddr.un.sun_path, tmp_name, sizeof(srcaddr.un.sun_path));
+                NCAT_INIT_SUN(&srcaddr, tmp_name);
                 free (tmp_name);
             }
 
@@ -1107,21 +1048,25 @@ int ncat_connect(void)
             nsock_pool_delete(mypool);
             return 1;
         }
-        /* Clear out whatever is left in the socket buffer which may be
-           already sent by proxy server along with http response headers. */
-        //line = socket_buffer_remainder(&stateful_buf, &n);
-        /* Write the leftover data to stdout. */
-        //Write(STDOUT_FILENO, line, n);
 
         /* Once the proxy negotiation is done, Nsock takes control of the
            socket. */
         cs.sock_nsi = nsock_iod_new2(mypool, connect_socket, NULL);
+        if (nsock_iod_set_hostname(cs.sock_nsi, o.sslservername) == -1)
+            bye("Failed to set hostname on iod.");
+        if (o.ssl)
+        {
+            /* connect_handler creates stdin_nsi and calls post_connect */
+            nsock_reconnect_ssl(mypool, cs.sock_nsi, connect_handler, o.conntimeout, NULL, NULL);
+        }
+        else
+        {
+            /* Create IOD for nsp->stdin */
+            if ((cs.stdin_nsi = nsock_iod_new2(mypool, 0, NULL)) == NULL)
+                bye("Failed to create stdin nsiod.");
 
-        /* Create IOD for nsp->stdin */
-        if ((cs.stdin_nsi = nsock_iod_new2(mypool, 0, NULL)) == NULL)
-            bye("Failed to create stdin nsiod.");
-
-        post_connect(mypool, cs.sock_nsi);
+            post_connect(mypool, cs.sock_nsi);
+        }
     }
 
     /* connect */
@@ -1133,7 +1078,7 @@ int ncat_connect(void)
         struct timeval end_time;
         double time;
         gettimeofday(&end_time, NULL);
-        time = TIMEVAL_MSEC_SUBTRACT(end_time, start_time) / 1000.0;
+        time = TIMEVAL_FSEC_SUBTRACT(end_time, start_time);
         loguser("%lu bytes sent, %lu bytes received in %.2f seconds.\n",
             nsock_iod_get_write_count(cs.sock_nsi),
             nsock_iod_get_read_count(cs.sock_nsi), time);
@@ -1227,13 +1172,11 @@ static void connect_handler(nsock_pool nsp, nsock_event evt, void *data)
                     : socket_strerror(nse_errorcode(evt)));
                 loguser("Trying next address...\n");
             }
-#ifdef HAVE_OPENSSL
-            /* If it's an SSL reconnect, clear out any old session info */
-            if (nsock_iod_check_ssl(cs.sock_nsi)) {
-              nsock_iod_delete(cs.sock_nsi, NSOCK_PENDING_NOTIFY);
-              cs.sock_nsi = new_iod(nsp);
-            }
-#endif
+            /* Delete the old IOD and make a new one for the next address.
+             * This also clears SSL session info. */
+            nsock_iod_delete(cs.sock_nsi, NSOCK_PENDING_NOTIFY);
+            cs.sock_nsi = new_iod(nsp);
+
             try_nsock_connect(nsp, next_addr);
             return;
         }
@@ -1254,7 +1197,7 @@ static void connect_handler(nsock_pool nsp, nsock_event evt, void *data)
     if (nsock_iod_check_ssl(cs.sock_nsi)) {
         /* Check the domain name. ssl_post_connect_check prints an
            error message if appropriate. */
-        if (!ssl_post_connect_check((SSL *)nsock_iod_get_ssl(cs.sock_nsi), o.target))
+        if (!ssl_post_connect_check((SSL *)nsock_iod_get_ssl(cs.sock_nsi), o.sslservername))
             bye("Certificate verification error.");
     }
 #endif
@@ -1321,8 +1264,16 @@ static void read_stdin_handler(nsock_pool nsp, nsock_event evt, void *data)
 
 
     if (status == NSE_STATUS_EOF) {
-        if (!o.noshutdown)
-            shutdown(nsock_iod_get_sd(cs.sock_nsi), SHUT_WR);
+        if (!o.noshutdown) {
+#ifdef HAVE_OPENSSL
+            SSL *ssl = NULL;
+            if (o.ssl && NULL != (ssl = (SSL *)nsock_iod_get_ssl(cs.sock_nsi))) {
+                SSL_shutdown(ssl);
+            }
+            else
+#endif
+                shutdown(nsock_iod_get_sd(cs.sock_nsi), SHUT_WR);
+        }
         /* In --send-only mode or non-TCP mode, exit after EOF on stdin. */
         if (o.proto != IPPROTO_TCP || (o.proto == IPPROTO_TCP && o.sendonly))
             nsock_loop_quit(nsp);
