@@ -1,9 +1,7 @@
 #include "nsock.h"
 #include "nmap_error.h"
 #include "NmapOps.h"
-#include "utils.h"
 #include "tcpip.h"
-#include "protocols.h"
 #include "libnetutil/netutil.h"
 
 #include "nse_main.h"
@@ -11,15 +9,15 @@
 
 #include "struct_ip.h"
 
-extern "C" {
-#include "lua.h"
-#include "lauxlib.h"
-#include "lualib.h"
-}
+#include "nse_lua.h"
 
 #include <assert.h>
 
 extern NmapOps o;
+#ifdef WIN32
+/* from libdnet's intf-win32.c */
+extern "C" int g_has_npcap_loopback;
+#endif
 
 enum {
   DNET_METATABLE = lua_upvalueindex(1),
@@ -32,6 +30,7 @@ typedef struct nse_dnet_udata
 {
   eth_t *eth;
   int sock; /* raw ip socket */
+  char devname[32]; /* libnetutil uses this len; dnet generally uses 16 */
 } nse_dnet_udata;
 
 static int l_dnet_new (lua_State *L)
@@ -64,7 +63,7 @@ static int l_dnet_get_interface_info (lua_State *L)
 
   nseU_setsfield(L, -1, "device", ii->devfullname);
   nseU_setsfield(L, -1, "shortname", ii->devname);
-  nseU_setnfield(L, -1, "netmask", ii->netmask_bits);
+  nseU_setifield(L, -1, "netmask", ii->netmask_bits);
 
   if (ii->addr.ss_family == AF_INET)
     inet_ntop(AF_INET, &((struct sockaddr_in *)&ii->addr)->sin_addr,
@@ -106,7 +105,7 @@ static int l_dnet_get_interface_info (lua_State *L)
   }
 
   nseU_setsfield(L, -1, "up", (ii->device_up ? "up" : "down"));
-  nseU_setnfield(L, -1, "mtu", ii->mtu);
+  nseU_setifield(L, -1, "mtu", ii->mtu);
 
   return 1;
 }
@@ -151,13 +150,27 @@ static eth_t *open_eth_cached (lua_State *L, int dnet_index, const char *device)
 static int ethernet_open (lua_State *L)
 {
   nse_dnet_udata *udata = (nse_dnet_udata *) nseU_checkudata(L, 1, DNET_METATABLE, "dnet");
-  const char *interface_name = luaL_checkstring(L, 2);
+  size_t len = 0;
+  const char *interface_name = luaL_checklstring(L, 2, &len);
+  if (len >= 32)
+    return luaL_argerror(L, 2, "device name too long");
   struct interface_info *ii = getInterfaceByName(interface_name, o.af());
 
-  if (ii == NULL || ii->device_type != devt_ethernet)
+  if (ii == NULL || ii->device_type != devt_ethernet
+#ifdef WIN32
+    && !(g_has_npcap_loopback && ii->device_type == devt_loopback)
+#endif
+    )
     return luaL_argerror(L, 2, "device is not valid ethernet interface");
 
   udata->eth = open_eth_cached(L, 1, interface_name);
+  strncpy(udata->devname, interface_name, len);
+  udata->devname[len] = '\0';
+  if (o.scriptTrace())
+  {
+      log_write(LOG_STDOUT, "%s: Ethernet open %s\n",
+          SCRIPT_ENGINE, udata->devname);
+  }
 
   return nseU_success(L);
 }
@@ -166,6 +179,12 @@ static int ethernet_close (lua_State *L)
 {
   nse_dnet_udata *udata = (nse_dnet_udata *) nseU_checkudata(L, 1, DNET_METATABLE, "dnet");
 
+  if (o.scriptTrace())
+  {
+      log_write(LOG_STDOUT, "%s: Ethernet close %s\n",
+          SCRIPT_ENGINE, udata->devname);
+  }
+  udata->devname[0] = '\0';
   udata->eth = NULL;
 
   lua_pushvalue(L, 1);
@@ -180,8 +199,18 @@ static int ethernet_send (lua_State *L)
   nse_dnet_udata *udata = (nse_dnet_udata *) nseU_checkudata(L, 1, DNET_METATABLE, "dnet");
   if (udata->eth == NULL)
     return luaL_error(L, "dnet ethernet interface is not open");
-  eth_send(udata->eth, luaL_checkstring(L, 2), lua_rawlen(L, 2));
-  return nseU_success(L);
+  size_t len = 0;
+  const char *frame = luaL_checklstring(L, 2, &len);
+  if (o.scriptTrace())
+  {
+      log_write(LOG_STDOUT, "%s: Ethernet frame (%lu bytes) > %s\n",
+          SCRIPT_ENGINE, len, udata->devname);
+  }
+  size_t sent = eth_send(udata->eth, frame, len);
+  if (sent == len)
+    return nseU_success(L);
+  else
+    return nseU_safeerror(L, "eth_send error: %lu", sent);
 }
 
 static int ip_open (lua_State *L)
@@ -191,6 +220,10 @@ static int ip_open (lua_State *L)
   if (udata->sock == -1)
     return luaL_error(L, "failed to open raw socket: %s (errno %d)",
         socket_strerror(socket_errno()), socket_errno());
+  if (o.scriptTrace())
+  {
+      log_write(LOG_STDOUT, "%s: raw IP socket open\n", SCRIPT_ENGINE);
+  }
   return nseU_success(L);
 }
 
@@ -201,6 +234,10 @@ static int ip_close (lua_State *L)
     return nseU_safeerror(L, "raw socket already closed");
   close(udata->sock);
   udata->sock = -1;
+  if (o.scriptTrace())
+  {
+      log_write(LOG_STDOUT, "%s: raw IP socket close\n", SCRIPT_ENGINE);
+  }
   return nseU_success(L);
 }
 
@@ -253,8 +290,13 @@ static int ip_send (lua_State *L)
 
     Strncpy(dev, route.ii.devname, sizeof(dev));
 
-    if (route.ii.device_type != devt_ethernet)
+    if (! (route.ii.device_type == devt_ethernet
+#ifdef WIN32
+          || (g_has_npcap_loopback && route.ii.device_type == devt_loopback)
+#endif
+          ) ) {
       goto usesock;
+    }
 
     /* above we fallback to using the raw socket if we can't find an (ethernet)
      * route to the host.  From here on out it's ethernet all the way.
@@ -265,16 +307,26 @@ static int ip_send (lua_State *L)
     else
       nexthop = &route.nexthop;
 
-    if (!getNextHopMAC(route.ii.devfullname, route.ii.mac, &hdr.src, nexthop, dstmac))
-      return luaL_error(L, "failed to determine next hop MAC address");
-
     /* Use cached ethernet device, and use udata's eth and interface to keep
      * track of if we're reusing the same device from the previous packet, and
      * close the cached device if not.
      */
     memset(&eth, 0, sizeof(eth));
-    memcpy(eth.srcmac, route.ii.mac, sizeof(eth.srcmac));
-    memcpy(eth.dstmac, dstmac, sizeof(eth.dstmac));
+
+#ifdef WIN32
+    // Only determine mac addr info if it's not the Npcap Loopback Adapter.
+    // Npcap loopback doesn't have a MAC address and isn't an ethernet device,
+    // so getNextHopMAC crashes.
+    if (!(g_has_npcap_loopback && route.ii.device_type == devt_loopback)) {
+#endif
+      if (!getNextHopMAC(route.ii.devfullname, route.ii.mac, &hdr.src, nexthop, dstmac))
+        return luaL_error(L, "failed to determine next hop MAC address");
+
+      memcpy(eth.srcmac, route.ii.mac, sizeof(eth.srcmac));
+      memcpy(eth.dstmac, dstmac, sizeof(eth.dstmac));
+#ifdef WIN32
+    } // end if not Npcap loopback
+#endif
 
     /* close any current ethernet associated with this userdata */
     lua_getfield(L, 1, "ethernet_close");
@@ -283,18 +335,23 @@ static int ip_send (lua_State *L)
 
     udata->eth = eth.ethsd = open_eth_cached(L, 1, route.ii.devname);
 
-    ret = send_ip_packet(udata->sock, &eth, &dst, (u8 *) packet, lua_rawlen(L, 2));
+    ret = send_ip_packet(udata->sock, &eth, &dst, (u8 *) packet, packetlen);
   } else {
 usesock:
 #ifdef WIN32
     if (strlen(dev) > 0)
       win32_fatal_raw_sockets(dev);
 #endif
-    ret = send_ip_packet(udata->sock, NULL, &dst, (u8 *) packet, lua_rawlen(L, 2));
+    ret = send_ip_packet(udata->sock, NULL, &dst, (u8 *) packet, packetlen);
   }
   if (ret == -1)
     return nseU_safeerror(L, "error while sending: %s (errno %d)",
         socket_strerror(socket_errno()), socket_errno());
+  if (o.scriptTrace())
+  {
+      log_write(LOG_STDOUT, "%s: RAW SEND %s\n",
+          SCRIPT_ENGINE, ippackethdrinfo((const u8 *)packet, packetlen, LOW_DETAIL));
+  }
 
   return nseU_success(L);
 }
