@@ -1,19 +1,17 @@
 /*
 * Binding for the libssh2 library. Note that there is not a one-to-one correspondance
 * between functions in libssh2 and the binding.
-* Currently, during the ssh2 handshake, a call to nsock.recieve may result in an EOF
+* Currently, during the ssh2 handshake, a call to nsock.receive may result in an EOF
 * error. This appears to only occur when stressing the ssh server (ie during a brute
 * force attempt) or while behind a restrictive firewall/IDS.
 * by Devin Bjelland
 */
 
 extern "C" {
-#include "lua.h"
-#include "lauxlib.h"
 #include "libssh2.h"
 }
+#include "nse_lua.h"
 
-#include "nse_debug.h"
 #include "nse_nsock.h"
 #include "nse_utility.h"
 
@@ -296,6 +294,7 @@ static int do_session_handshake (lua_State *L, int status, lua_KContext ctx) {
 
     if (rc) {
         libssh2_session_free(sshu->session);
+        sshu->session = NULL;
         return luaL_error(L, "Unable to complete libssh2 handshake.");
     }
 
@@ -479,12 +478,34 @@ static int userauth_list (lua_State *L, int status, lua_KContext ctx) {
 }
 
 /*
-* Returns list of supported authenication methods
+* Returns list of supported authentication methods
 */
 static int l_userauth_list (lua_State *L) {
     return userauth_list(L, 0, 0);
 }
 
+static int userauth_banner (lua_State *L, int status, lua_KContext ctx) {
+    char *auth_banner = NULL;
+    struct ssh_userdata *state = NULL;
+
+    state = (struct ssh_userdata *) nseU_checkudata(L, 1, SSH2_UDATA, "ssh2");
+    assert(state->session != NULL);
+
+    if (LIBSSH2_ERROR_NONE == libssh2_userauth_banner(state->session, &auth_banner))
+    {
+      lua_pushstring(L, auth_banner);
+      return 1;
+    }
+    return 0;
+}
+
+/*
+* Returns pre-auth banner
+*/
+static int l_userauth_banner (lua_State *L) {
+    return userauth_banner(L, 0, 0);
+}
+
 static int userauth_publickey (lua_State *L, int status, lua_KContext ctx) {
     int rc;
     const char *username, *private_key_file, *passphrase, *public_key_file;
@@ -879,6 +900,7 @@ static const struct luaL_Reg libssh2[] = {
     { "session_open", l_session_open },
     { "hostkey_hash", l_hostkey_hash },
     { "set_timeout", l_set_timeout },
+    { "userauth_banner", l_userauth_banner },
     { "userauth_list", l_userauth_list },
     { "userauth_publickey", l_userauth_publickey },
     { "read_publickey", l_read_publickey },
