@@ -25,16 +25,13 @@
 #include <stdlib.h>
 #include <string.h>
 
-extern "C" {
-#include "lua.h"
-#include "lauxlib.h"
-}
+#include "nse_lua.h"
 
 #include <zlib.h>
 
 /*
 ** =========================================================================
-** compile time options wich determine available functionality
+** compile time options which determine available functionality
 ** =========================================================================
 */
 
@@ -166,7 +163,7 @@ static lz_stream *lzstream_check(lua_State *L, int index, int state) {
 
 static int lzstream_tostring(lua_State *L) {
     lz_stream *s = (lz_stream*)luaL_checkudata(L, 1, ZSTREAMMETA);
-    if (s == NULL) luaL_argerror(L, 1, "bad zlib stream");
+    if (s == NULL) return luaL_argerror(L, 1, "bad zlib stream");
 
     if (s->state == LZ_NONE) {
         lua_pushstring(L, "zlib stream (closed)");
@@ -596,16 +593,16 @@ static int lzstream_decompress(lua_State *L) {
 
 static int lzstream_readline(lua_State *L) {
     lz_stream *s;
-    int sucess;
+    int success;
 
     s = lzstream_check(L, lua_upvalueindex(1), LZ_INFLATE);
-    sucess = lz_read_line(L, s);
+    success = lz_read_line(L, s);
 
     if (s->error != Z_OK) {
         return lz_pushresult(L, s);
     }
 
-    if (sucess) {
+    if (success) {
         return 1;
     } else {
         /* EOF */
@@ -964,8 +961,12 @@ LUALIB_API int luaopen_zlib(lua_State *L)
 
     PUSH_LITERAL(FILTERED)
     PUSH_LITERAL(HUFFMAN_ONLY)
+#ifdef Z_RLE
     PUSH_LITERAL(RLE)
+#endif
+#ifdef Z_FIXED
     PUSH_LITERAL(FIXED)
+#endif
     PUSH_LITERAL(DEFAULT_STRATEGY)
 
     PUSH_NUMBER(MINIMUM_MEMLEVEL, 1)
