@@ -3,91 +3,62 @@
  * maintained internally by Nmap.                                          *
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
- *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2011 Insecure.Com LLC. Nmap is    *
- * also a registered trademark of Insecure.Com LLC.  This program is free  *
- * software; you may redistribute and/or modify it under the terms of the  *
- * GNU General Public License as published by the Free Software            *
- * Foundation; Version 2 with the clarifications and exceptions described  *
- * below.  This guarantees your right to use, modify, and redistribute     *
- * this software under certain conditions.  If you wish to embed Nmap      *
- * technology into proprietary software, we sell alternative licenses      *
- * (contact sales@insecure.com).  Dozens of software vendors already       *
- * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, and version detection.                                       *
- *                                                                         *
- * Note that the GPL places important restrictions on "derived works", yet *
- * it does not provide a detailed definition of that term.  To avoid       *
- * misunderstandings, we consider an application to constitute a           *
- * "derivative work" for the purpose of this license if it does any of the *
- * following:                                                              *
- * o Integrates source code from Nmap                                      *
- * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-db or nmap-service-probes.                                    *
- * o Executes Nmap and parses the results (as opposed to typical shell or  *
- *   execution-menu apps, which simply display raw Nmap output and so are  *
- *   not derivative works.)                                                *
- * o Integrates/includes/aggregates Nmap into a proprietary executable     *
- *   installer, such as those produced by InstallShield.                   *
- * o Links to a library or executes a program that does any of the above   *
- *                                                                         *
- * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap.  This list is not exclusive, but is meant to clarify our *
- * interpretation of derived works with some common examples.  Our         *
- * interpretation applies only to Nmap--we don't speak for other people's  *
- * GPL works.                                                              *
- *                                                                         *
- * If you have any questions about the GPL licensing restrictions on using *
- * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
- * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to dozens  *
- * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates as well as helping to     *
- * fund the continued development of Nmap technology.  Please email        *
- * sales@insecure.com for further information.                             *
- *                                                                         *
- * As a special exception to the GPL terms, Insecure.Com LLC grants        *
- * permission to link the code of this program with any version of the     *
- * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
- * linked combinations including the two. You must obey the GNU GPL in all *
- * respects for all of the code used other than OpenSSL.  If you modify    *
- * this file, you may extend this exception to your version of the file,   *
- * but you are not obligated to do so.                                     *
- *                                                                         *
- * If you received these files with a written license agreement or         *
- * contract stating terms other than the terms above, then that            *
- * alternative license agreement takes precedence over these comments.     *
- *                                                                         *
- * Source is provided to this software because we believe users have a     *
- * right to know exactly what a program is going to do before they run it. *
- * This also allows you to audit the software for security holes (none     *
- * have been found so far).                                                *
- *                                                                         *
- * Source code also allows you to port Nmap to new platforms, fix bugs,    *
- * and add new features.  You are highly encouraged to send your changes   *
- * to nmap-dev@insecure.org for possible incorporation into the main       *
- * distribution.  By sending these changes to Fyodor or one of the         *
- * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
- * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because the *
- * inability to relicense code has caused devastating problems for other   *
- * Free Software projects (such as KDE and NASM).  We also occasionally    *
- * relicense the code to third parties as discussed above.  If you wish to *
- * specify special license conditions of your contributions, just say so   *
- * when you send them.                                                     *
- *                                                                         *
- * This program is distributed in the hope that it will be useful, but     *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License v2.0 for more details at                         *
- * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
- * included with Nmap.                                                     *
- *                                                                         *
+ *
+ * The Nmap Security Scanner is (C) 1996-2024 Nmap Software LLC ("The Nmap
+ * Project"). Nmap is also a registered trademark of the Nmap Project.
+ *
+ * This program is distributed under the terms of the Nmap Public Source
+ * License (NPSL). The exact license text applying to a particular Nmap
+ * release or source code control revision is contained in the LICENSE
+ * file distributed with that version of Nmap or source code control
+ * revision. More Nmap copyright/legal information is available from
+ * https://nmap.org/book/man-legal.html, and further information on the
+ * NPSL license itself can be found at https://nmap.org/npsl/ . This
+ * header summarizes some key points from the Nmap license, but is no
+ * substitute for the actual license text.
+ *
+ * Nmap is generally free for end users to download and use themselves,
+ * including commercial use. It is available from https://nmap.org.
+ *
+ * The Nmap license generally prohibits companies from using and
+ * redistributing Nmap in commercial products, but we sell a special Nmap
+ * OEM Edition with a more permissive license and special features for
+ * this purpose. See https://nmap.org/oem/
+ *
+ * If you have received a written Nmap license agreement or contract
+ * stating terms other than these (such as an Nmap OEM license), you may
+ * choose to use and redistribute Nmap under those terms instead.
+ *
+ * The official Nmap Windows builds include the Npcap software
+ * (https://npcap.com) for packet capture and transmission. It is under
+ * separate license terms which forbid redistribution without special
+ * permission. So the official Nmap Windows builds may not be redistributed
+ * without special permission (such as an Nmap OEM license).
+ *
+ * Source is provided to this software because we believe users have a
+ * right to know exactly what a program is going to do before they run it.
+ * This also allows you to audit the software for security holes.
+ *
+ * Source code also allows you to port Nmap to new platforms, fix bugs, and
+ * add new features. You are highly encouraged to submit your changes as a
+ * Github PR or by email to the dev@nmap.org mailing list for possible
+ * incorporation into the main distribution. Unless you specify otherwise, it
+ * is understood that you are offering us very broad rights to use your
+ * submissions as described in the Nmap Public Source License Contributor
+ * Agreement. This is important because we fund the project by selling licenses
+ * with various terms, and also because the inability to relicense code has
+ * caused devastating problems for other Free Software projects (such as KDE
+ * and NASM).
+ *
+ * The free version of Nmap is distributed in the hope that it will be
+ * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,
+ * indemnification and commercial support are all available through the
+ * Npcap OEM program--see https://nmap.org/oem/
+ *
  ***************************************************************************/
 
-/* $Id: portlist.cc 21904 2011-01-21 00:04:16Z fyodor $ */
+/* $Id: portlist.cc 38848 2024-04-18 16:23:04Z dmiller $ */
 
 
 #include "nmap.h"
@@ -96,12 +67,10 @@
 #include "NmapOps.h"
 #include "services.h"
 #include "protocols.h"
-#include "nmap_rpc.h"
 #include "tcpip.h"
+#include "string_pool.h"
 #include "libnetutil/netutil.h"
 
-using namespace std;
-
 #if HAVE_STRINGS_H
 #include <strings.h>
 #endif /* HAVE_STRINGS_H */
@@ -115,26 +84,50 @@ Port::Port() {
   state_reason_init(&reason);
 }
 
-void Port::freeService() {
-  if (service != NULL) {
-    if (service->name)
-      free(service->name);
-    if (service->product)
-      free(service->product);
-    if (service->version)
-      free(service->version);
-    if (service->extrainfo)
-      free(service->extrainfo);
-    if (service->hostname)
-      free(service->hostname);
-    if (service->ostype)
-      free(service->ostype);
-    if (service->devicetype)
-      free(service->devicetype);
-    if (service->service_fp)
-      free(service->service_fp);
-    delete service;
-  }
+void serviceDeductions::erase() {
+  std::vector<char *>::iterator it;
+
+  if (this->product)
+    free(this->product);
+  if (this->version)
+    free(this->version);
+  if (this->extrainfo)
+    free(this->extrainfo);
+  if (this->hostname)
+    free(this->hostname);
+  if (this->ostype)
+    free(this->ostype);
+  if (this->devicetype)
+    free(this->devicetype);
+  if (this->service_fp)
+    free(this->service_fp);
+  // For now, always free CPE strings
+  for (it = this->cpe.begin(); it != this->cpe.end(); it++)
+    free(*it);
+  this->cpe.clear();
+
+  this->name = NULL;
+  this->name_confidence = 0;
+  this->product = NULL;
+  this->version = NULL;
+  this->extrainfo = NULL;
+  this->hostname = NULL;
+  this->ostype = NULL;
+  this->devicetype = NULL;
+  this->service_tunnel = SERVICE_TUNNEL_NONE;
+  this->service_fp = NULL;
+  this->dtype = SERVICE_DETECTION_TABLE;
+}
+
+void Port::freeScriptResults(void)
+{
+#ifndef NOLUA
+    for (ScriptResults::iterator it = scriptResults.begin();
+        it != scriptResults.end(); it++) {
+      delete (*it);
+    }
+    scriptResults.clear();
+#endif
 }
 
 /* Fills in namebuf (as long as there is space in buflen) with the
@@ -154,9 +147,9 @@ void Port::getNmapServiceName(char *namebuf, int buflen) const {
   if (service != NULL && service->name != NULL) {
     service_name = service->name;
   } else {
-    struct servent *service;
+    const struct nservent *service;
 
-    service = nmap_getservbyport(htons(portno), IPPROTO2STR(proto));
+    service = nmap_getservbyport(portno, proto);
     if (service != NULL)
       service_name = service->s_name;
     else
@@ -172,8 +165,11 @@ void Port::getNmapServiceName(char *namebuf, int buflen) const {
   } else {
     len = Snprintf(namebuf, buflen, "%sunknown", tunnel_prefix);
   }
-  if (len >= buflen || len < 0)
+  if (len >= buflen || len < 0) {
     namebuf[0] = '\0';
+    return;
+  }
+
 }
 
 serviceDeductions::serviceDeductions() {
@@ -188,10 +184,6 @@ serviceDeductions::serviceDeductions() {
   service_tunnel = SERVICE_TUNNEL_NONE;
   service_fp = NULL;
   dtype = SERVICE_DETECTION_TABLE;
-  rpc_status = RPC_STATUS_UNTESTED;
-  rpc_program = 0;
-  rpc_lowver = 0;
-  rpc_highver = 0;
 }
 
 // Uses the sd->{product,version,extrainfo} if available to fill
@@ -228,7 +220,7 @@ void serviceDeductions::populateFullVersionString(char *buf, size_t n) const {
       strncat(dst, " ", spaceleft);
       spaceleft--;
     }
-    
+
     if (spaceleft < strlen(version)) {
       strncat(dst, version, spaceleft - 3);
       strncat(dst, "...", spaceleft);
@@ -246,7 +238,7 @@ void serviceDeductions::populateFullVersionString(char *buf, size_t n) const {
       strncat(dst, " ", spaceleft);
       spaceleft--;
     }
-    // This time we need to trucate inside of the () so we have spaceleft - 2
+    // This time we need to truncate inside of the () so we have spaceleft - 2
     strncat(dst, "(", spaceleft);
     if (spaceleft - 2 < strlen(extrainfo)) {
       strncat(dst, extrainfo, spaceleft - 5);
@@ -267,17 +259,16 @@ void serviceDeductions::populateFullVersionString(char *buf, size_t n) const {
 // pass in an allocated struct serviceDeductions (don't worry about
 // initializing, and you don't have to free any internal ptrs.  See the
 // serviceDeductions definition for the fields that are populated.
-// Returns 0 if at least a name is available.
-const void PortList::getServiceDeductions(u16 portno, int protocol, struct serviceDeductions *sd) const {
+void PortList::getServiceDeductions(u16 portno, int protocol, struct serviceDeductions *sd) const {
   const Port *port;
 
   port = lookupPort(portno, protocol);
   if (port == NULL || port->service == NULL) {
-    struct servent *service;
+    const struct nservent *service;
 
     /* Look up the service name. */
     *sd = serviceDeductions();
-    service = nmap_getservbyport(htons(portno), IPPROTO2STR(protocol));
+    service = nmap_getservbyport(portno, protocol);
     if (service != NULL)
       sd->name = service->s_name;
     else
@@ -305,14 +296,14 @@ static char *cstringSanityCheck(const char* string, int len) {
   int slen;
 
   if(!string)
-	  return NULL;
+          return NULL;
 
   slen = strlen(string);
   if (slen > len) slen = len;
   result = (char *) safe_malloc(slen + 1);
   memcpy(result, string, slen);
   result[slen] = '\0';
-  replacenonprintable(result, slen, '.'); 
+  replacenonprintable(result, slen, '.');
   return result;
 }
 
@@ -320,12 +311,17 @@ void PortList::setServiceProbeResults(u16 portno, int protocol,
   enum serviceprobestate sres, const char *sname,
   enum service_tunnel_type tunnel, const char *product, const char *version,
   const char *extrainfo, const char *hostname, const char *ostype,
-  const char *devicetype, const char *fingerprint) {
+  const char *devicetype, const std::vector<const char *> *cpe,
+  const char *fingerprint) {
+  std::vector<char *>::iterator it;
   Port *port;
+  char *p;
 
   port = createPort(portno, protocol);
   if (port->service == NULL)
     port->service = new serviceDeductions;
+  else
+    port->service->erase();
 
   if (sres == PROBESTATE_FINISHED_HARDMATCHED
       || sres == PROBESTATE_FINISHED_SOFTMATCHED) {
@@ -340,8 +336,8 @@ void PortList::setServiceProbeResults(u16 portno, int protocol,
     /* PROBESTATE_FINISHED_NOMATCH, PROBESTATE_EXCLUDED, PROBESTATE_INCOMPLETE.
        Just look up the service name if none is provided. */
     if (sname == NULL) {
-      struct servent *service;
-      service = nmap_getservbyport(htons(portno), IPPROTO2STR(protocol));
+      const struct nservent *service;
+      service = nmap_getservbyport(portno, protocol);
       if (service != NULL)
         sname = service->s_name;
     }
@@ -352,14 +348,14 @@ void PortList::setServiceProbeResults(u16 portno, int protocol,
   // port->serviceprobe_results = sres;
   port->service->service_tunnel = tunnel;
 
-  if (sname) 
-    port->service->name = strdup(sname);
+  if (sname)
+    port->service->name = string_pool_insert(sname);
   else
     port->service->name = NULL;
 
-  if (fingerprint) 
+  if (fingerprint)
     port->service->service_fp = strdup(fingerprint);
-  else 
+  else
     port->service->service_fp = NULL;
 
   port->service->product = cstringSanityCheck(product, 80);
@@ -368,60 +364,26 @@ void PortList::setServiceProbeResults(u16 portno, int protocol,
   port->service->hostname = cstringSanityCheck(hostname, 80);
   port->service->ostype = cstringSanityCheck(ostype, 32);
   port->service->devicetype = cstringSanityCheck(devicetype, 32);
-}
 
-/* Sets the results of an RPC scan.  if rpc_status is not
-   RPC_STATUS_GOOD_PROGRAM, pass 0 for the other args.  This function
-   takes care of setting the port's service and version appropriately. */
-void PortList::setRPCProbeResults(u16 portno, int proto, int rpcs, unsigned long rpcp, 
-			unsigned int rpcl, unsigned int rpch) {
-  Port *port;
-  const char *newsvc;
-  char verbuf[128];
+  if (cpe) {
+    std::vector<const char *>::const_iterator cit;
 
-  port = createPort(portno, proto);
-  if (port->service == NULL)
-    port->service = new serviceDeductions;
-
-  port->service->rpc_status = rpcs;
-  if (port->service->rpc_status == RPC_STATUS_GOOD_PROG) {
-    port->service->rpc_program = rpcp;
-    port->service->rpc_lowver = rpcl;
-    port->service->rpc_highver = rpch;
-
-    // Now set the service/version info
-    newsvc = nmap_getrpcnamebynum(rpcp);
-    if (!newsvc) newsvc = "rpc.unknownprog"; // should never happen
-    if (port->service->name)
-      free(port->service->name);
-    port->service->name = strdup(newsvc);
-    if (port->service->rpc_lowver == port->service->rpc_highver)
-      Snprintf(verbuf, sizeof(verbuf), "%i", port->service->rpc_lowver);
-    else
-      Snprintf(verbuf, sizeof(verbuf), "%i-%i", port->service->rpc_lowver, port->service->rpc_highver);
-    port->service->version = strdup(verbuf);
-    Snprintf(verbuf, sizeof(verbuf), "rpc #%li", port->service->rpc_program);
-    port->service->extrainfo = strdup(verbuf);
-    port->service->name_confidence = 10;
-    port->service->dtype = SERVICE_DETECTION_PROBED;
-  } else if (port->service->rpc_status == RPC_STATUS_UNKNOWN) {
-    if (port->service->name)
-      free(port->service->name);
-    
-    port->service->name = strdup("rpc.unknown");
-    port->service->name_confidence = 8;
-    port->service->dtype = SERVICE_DETECTION_PROBED;
+    for (cit = cpe->begin(); cit != cpe->end(); cit++) {
+      p = cstringSanityCheck(*cit, 80);
+      if (p != NULL)
+        port->service->cpe.push_back(p);
+    }
   }
 }
 
 
 #ifndef NOLUA
-void PortList::addScriptResult(u16 portno, int protocol, ScriptResult& sr) {
+void PortList::addScriptResult(u16 portno, int protocol, ScriptResult *sr) {
   Port *port;
 
   port = createPort(portno, protocol);
 
-  port->scriptResults.push_back(sr);
+  port->scriptResults.insert(sr);
 }
 #endif
 
@@ -461,17 +423,22 @@ PortList::PortList() {
 PortList::~PortList() {
   int proto, i;
 
-  if (idstr) { 
+  if (idstr) {
     free(idstr);
     idstr = NULL;
   }
 
   for(proto=0; proto < PORTLIST_PROTO_MAX; proto++) { // for every protocol
     if(port_list[proto]) {
-      for(i=0; i < port_list_count[proto]; i++) { // free every Port 
-        if(port_list[proto][i]) {
-          port_list[proto][i]->freeService();
-          delete port_list[proto][i];
+      for(i=0; i < port_list_count[proto]; i++) { // free every Port
+        Port *port = port_list[proto][i];
+        if(port) {
+          if (port->service) {
+            port->service->erase();
+            delete port->service;
+          }
+          port->freeScriptResults();
+          delete port;
         }
       }
       free(port_list[proto]);
@@ -493,8 +460,7 @@ void PortList::setDefaultPortState(u8 protocol, int state) {
   default_port_state[proto].state = state;
 }
 
-void PortList::setPortState(u16 portno, u8 protocol, int state) {
-  const Port *oldport;
+void PortList::setPortState(u16 portno, u8 protocol, int state, int *oldstate) {
   Port *current;
   int proto = INPROTO2PORTLISTPROTO(protocol);
 
@@ -502,38 +468,39 @@ void PortList::setPortState(u16 portno, u8 protocol, int state) {
 
   if ((state == PORT_OPEN && o.verbose) || (o.debugging > 1)) {
     log_write(LOG_STDOUT, "Discovered %s port %hu/%s%s\n",
-	      statenum2str(state), portno, 
-	      proto2ascii_lowercase(protocol), idstr? idstr : "");
+              statenum2str(state), portno,
+              proto2ascii_lowercase(protocol), idstr? idstr : "");
     log_flush(LOG_STDOUT);
   }
 
 
   /* Make sure state is OK */
   if (state != PORT_OPEN && state != PORT_CLOSED && state != PORT_FILTERED &&
-      state != PORT_UNFILTERED && state != PORT_OPENFILTERED && 
+      state != PORT_UNFILTERED && state != PORT_OPENFILTERED &&
       state != PORT_CLOSEDFILTERED)
     fatal("%s: attempt to add port number %d with illegal state %d\n", __func__, portno, state);
 
   assert(protocol!=IPPROTO_IP || portno<256);
 
-  oldport = lookupPort(portno, protocol);
-  if (oldport != NULL) {
-    /* We must discount our statistics from the old values.  Also warn
-       if a complete duplicate */
-    if (o.debugging && oldport->state == state) {
-      error("Duplicate port (%hu/%s)", portno, proto2ascii_lowercase(protocol));
-    } 
-    state_counts_proto[proto][oldport->state]--;
+  bool created = false;
+  current = createPort(portno, protocol, &created);
+
+  if (!created) {
+    /* We must discount our statistics from the old values.
+     * Duplicates are not a problem and are expected due to optimistic state
+     * setting in ultrascan_port_pspec_update() */
+    state_counts_proto[proto][current->state]--;
+    if (oldstate) *oldstate = current->state;
   } else {
     state_counts_proto[proto][default_port_state[proto].state]--;
+    if (oldstate) *oldstate = PORT_TESTING;
   }
-  current = createPort(portno, protocol);
-  
+
   current->state = state;
   state_counts_proto[proto][state]++;
- 
+
   if(state == PORT_FILTERED || state == PORT_OPENFILTERED)
-  	setStateReason(portno, protocol, ER_NORESPONSE, 0, 0); 
+    setStateReason(current, ER_NORESPONSE, 0, NULL);
   return;
 }
 
@@ -548,7 +515,7 @@ int PortList::getPortState(u16 portno, u8 protocol) {
 }
 
 /* Return true if nothing special is known about this port; i.e., it's in the
-   default state as defiend by setDefaultPortState and every other data field is
+   default state as defined by setDefaultPortState and every other data field is
    unset. */
 bool PortList::portIsDefault(u16 portno, u8 protocol) {
   return lookupPort(portno, protocol) == NULL;
@@ -591,14 +558,14 @@ int PortList::getStateCounts(int state) const {
    except that if you ask for both TCP, UDP & SCTP, every TCP port
    will be returned before we start returning UDP and SCTP ports */
 Port *PortList::nextPort(const Port *cur, Port *next,
-			 int allowed_protocol, int allowed_state) {
+                         int allowed_protocol, int allowed_state) const {
   int proto;
   int mapped_pno;
   Port *port;
-  
+
   if (cur) {
     proto = INPROTO2PORTLISTPROTO(cur->proto);
-    assert(port_map[proto]!=NULL); // Hmm, it's not posible to handle port that doesn't have anything in map
+    assert(port_map[proto]!=NULL); // Hmm, it's not possible to handle port that doesn't have anything in map
     assert(cur->proto!=IPPROTO_IP || cur->portno<256);
     mapped_pno = port_map[proto][cur->portno];
     mapped_pno++; //  we're interested in next port after current
@@ -611,7 +578,7 @@ Port *PortList::nextPort(const Port *cur, Port *next,
       proto = INPROTO2PORTLISTPROTO(allowed_protocol);
     mapped_pno = 0;
   }
-  
+
   if(port_list[proto] != NULL) {
     for(;mapped_pno < port_list_count[proto]; mapped_pno++) {
       port = port_list[proto][mapped_pno];
@@ -626,7 +593,7 @@ Port *PortList::nextPort(const Port *cur, Port *next,
       }
     }
   }
-  
+
   /* if all protocols, than after TCP search UDP & SCTP */
   if((!cur && allowed_protocol == TCPANDUDPANDSCTP) ||
       (cur && proto == INPROTO2PORTLISTPROTO(IPPROTO_TCP)))
@@ -636,13 +603,13 @@ Port *PortList::nextPort(const Port *cur, Port *next,
   if((!cur && allowed_protocol == UDPANDSCTP) ||
       (cur && proto == INPROTO2PORTLISTPROTO(IPPROTO_UDP)))
     return(nextPort(NULL, next, IPPROTO_SCTP, allowed_state));
-  
-  return(NULL); 
+
+  return(NULL);
 }
 
 /* Convert portno and protocol into the internal indices used to index
-   port_list. Returns false on error, true otherwise. */
-bool PortList::mapPort(u16 *portno, u8 *protocol) const {
+   port_list. */
+void PortList::mapPort(u16 *portno, u8 *protocol) const {
   int mapped_portno, mapped_protocol;
 
   mapped_protocol = INPROTO2PORTLISTPROTO(*protocol);
@@ -650,37 +617,31 @@ bool PortList::mapPort(u16 *portno, u8 *protocol) const {
   if (*protocol == IPPROTO_IP)
     assert(*portno < 256);
   if(port_map[mapped_protocol]==NULL || port_list[mapped_protocol]==NULL) {
-    assert(0);
     fatal("%s(%i,%i): you're trying to access uninitialized protocol", __func__, *portno, *protocol);
   }
   mapped_portno = port_map[mapped_protocol][*portno];
 
   assert(mapped_portno < port_list_count[mapped_protocol]);
   assert(mapped_portno >= 0);
-  
+
   *portno = mapped_portno;
   *protocol = mapped_protocol;
-
-  return true;
 }
 
 const Port *PortList::lookupPort(u16 portno, u8 protocol) const {
-  if (!mapPort(&portno, &protocol))
-    return NULL;
-
+  mapPort(&portno, &protocol);
   return port_list[protocol][portno];
 }
 
 /* Create the port if it doesn't exist; otherwise this is like lookupPort. */
-Port *PortList::createPort(u16 portno, u8 protocol) {
+Port *PortList::createPort(u16 portno, u8 protocol, bool *created) {
   Port *p;
   u16 mapped_portno;
   u8 mapped_protocol;
 
   mapped_portno = portno;
   mapped_protocol = protocol;
-  if (!mapPort(&mapped_portno, &mapped_protocol))
-    return NULL;
+  mapPort(&mapped_portno, &mapped_protocol);
 
   p = port_list[mapped_protocol][mapped_portno];
   if (p == NULL) {
@@ -690,9 +651,11 @@ Port *PortList::createPort(u16 portno, u8 protocol) {
     p->state = default_port_state[mapped_protocol].state;
     p->reason.reason_id = ER_NORESPONSE;
     port_list[mapped_protocol][mapped_portno] = p;
+    if (created) *created = true;
   }
+  else if (created) *created = false;
 
-  return port_list[mapped_protocol][mapped_portno];
+  return p;
 }
 
 int PortList::forgetPort(u16 portno, u8 protocol) {
@@ -700,8 +663,7 @@ int PortList::forgetPort(u16 portno, u8 protocol) {
 
   log_write(LOG_PLAIN, "Removed %d\n", portno);
 
-  if (!mapPort(&portno, &protocol))
-    return -1;
+  mapPort(&portno, &protocol);
 
   answer = port_list[protocol][portno];
   if (answer == NULL)
@@ -709,22 +671,21 @@ int PortList::forgetPort(u16 portno, u8 protocol) {
 
   state_counts_proto[protocol][answer->state]--;
   state_counts_proto[protocol][default_port_state[protocol].state]++;
-    
-  delete answer;
 
   port_list[protocol][portno] = NULL;
 
-  if (o.verbose) {  
+  if (o.verbose) {
     log_write(LOG_STDOUT, "Deleting port %hu/%s, which we thought was %s\n",
-	      portno, proto2ascii_lowercase(answer->proto),
-	      statenum2str(answer->state));
+              portno, proto2ascii_lowercase(answer->proto),
+              statenum2str(answer->state));
     log_flush(LOG_STDOUT);
-  }    
+  }
 
+  delete answer;
   return 0;
 }
 
-/* Just free memory used by PortList::port_map[]. Should be done somewhere 
+/* Just free memory used by PortList::port_map[]. Should be done somewhere
  * before closing nmap. */
 void PortList::freePortMap() {
   int proto;
@@ -741,31 +702,31 @@ void PortList::freePortMap() {
     port_list_count[proto] = 0;
   }
 }
-  
+
 
 u16 *PortList::port_map[PORTLIST_PROTO_MAX];
 u16 *PortList::port_map_rev[PORTLIST_PROTO_MAX];
 int PortList::port_list_count[PORTLIST_PROTO_MAX];
 
-/* This function must be runned before any PortList object is created.
- * It must be runned for every used protocol. The data in "ports" 
+/* This function must be run before any PortList object is created.
+ * It must be run for every used protocol. The data in "ports"
  * should be sorted. */
 void PortList::initializePortMap(int protocol, u16 *ports, int portcount) {
   int i;
   int ports_max = (protocol == IPPROTO_IP) ? 256 : 65536;
   int proto = INPROTO2PORTLISTPROTO(protocol);
-  
+
   if (port_map[proto] != NULL || port_map_rev[proto] != NULL)
     fatal("%s: portmap for protocol %i already initialized", __func__, protocol);
 
   assert(port_list_count[proto]==0);
-  
+
   /* this memory will never be freed, but this is the way it has to be. */
   port_map[proto] = (u16 *) safe_zalloc(sizeof(u16) * ports_max);
   port_map_rev[proto] = (u16 *) safe_zalloc(sizeof(u16) * portcount);
 
   port_list_count[proto] = portcount;
-  
+
   for(i=0; i < portcount; i++) {
     port_map[proto][ports[i]] = i;
     port_map_rev[proto][i] = ports[i];
@@ -777,19 +738,26 @@ void PortList::initializePortMap(int protocol, u16 *ports, int portcount) {
 
   /* Cycles through the 0 or more "ignored" ports which should be
    consolidated for Nmap output.  They are returned sorted by the
-   number of prots in the state, starting with the most common.  It
+   number of ports in the state, starting with the most common.  It
    should first be called with PORT_UNKNOWN to obtain the most popular
    ignored state (if any).  Then call with that state to get the next
    most popular one.  Returns the state if there is one, but returns
    PORT_UNKNOWN if there are no (more) states which qualify for
    consolidation */
-int PortList::nextIgnoredState(int prevstate) {
+int PortList::nextIgnoredState(int prevstate) const {
 
   int beststate = PORT_UNKNOWN;
-  
+  int count = 0;
+  int prevcount = 0;
+  int bestcount = 0;
+
+  if (prevstate != PORT_UNKNOWN) {
+    prevcount = getStateCounts(prevstate);
+  }
+
   for(int state=0; state < PORT_HIGHEST_STATE; state++) {
     /* The state must be ignored */
-    if (!isIgnoredState(state)) 
+    if (!isIgnoredState(state, &count))
       continue;
 
     /* We can't give the same state again ... */
@@ -797,25 +765,29 @@ int PortList::nextIgnoredState(int prevstate) {
 
     /* If a previous state was given, we must have fewer ports than
        that one, or be tied but be a larger state number */
-    if (prevstate != PORT_UNKNOWN && 
-	(getStateCounts(state) > getStateCounts(prevstate) ||
-	 (getStateCounts(state) == getStateCounts(prevstate) && state <= prevstate)))
+    if (prevstate != PORT_UNKNOWN &&
+        (count > prevcount ||
+         (count == prevcount && state <= prevstate)))
       continue;
 
     /* We only qualify if we have more ports than the current best */
-    if (beststate != PORT_UNKNOWN && getStateCounts(beststate) >= getStateCounts(state))
+    if (beststate != PORT_UNKNOWN && bestcount >= count)
       continue;
 
     /* Yay!  We found the best state so far ... */
     beststate = state;
+    bestcount = count;
   }
 
   return beststate;
 }
 
-/* Returns true if a state should be ignored (consolidated), false otherwise */
-bool PortList::isIgnoredState(int state) {
+/* Returns true if a state should be ignored (consolidated), false otherwise.
+ * If result is true and count is provided, it will be filled with the count of
+ * ports in that state. */
+bool PortList::isIgnoredState(int state, int *count) const {
 
+  int tmp_count = 0;
   if (o.debugging > 2)
     return false;
 
@@ -826,42 +798,47 @@ bool PortList::isIgnoredState(int state) {
   if (state == PORT_OPENFILTERED && (o.verbose > 2 || o.debugging > 2))
     return false;
 
+  tmp_count = getStateCounts(state);
+  if (count != NULL) {
+    *count = tmp_count;
+  }
   /* If openonly, we always ignore states that don't at least have open
      as a possibility. */
-  if (o.openOnly() && state != PORT_OPENFILTERED && state != PORT_UNFILTERED 
-      && getStateCounts(state) > 0)
+  if (o.openOnly() && state != PORT_OPENFILTERED && state != PORT_UNFILTERED
+      && tmp_count > 0)
     return true;
 
   int max_per_state = 25; // Ignore states with more ports than this
   /* We will show more ports when verbosity is requested */
   if (o.verbose || o.debugging) {
     if (o.ipprotscan)
-      max_per_state *= (o.verbose + 3 * o.debugging);
+      max_per_state *= ((o.verbose + 1) + 3 * o.debugging);
     else
-      max_per_state *= (o.verbose + 20 * o.debugging);
+      max_per_state *= ((o.verbose + 1) + 20 * o.debugging);
   }
 
-  if (getStateCounts(state) > max_per_state)
+  if (tmp_count > max_per_state)
     return true;
 
   return false;
 }
 
-int PortList::numIgnoredStates() {
+int PortList::numIgnoredStates() const {
   int numstates = 0;
   for(int state=0; state < PORT_HIGHEST_STATE; state++) {
-    if (isIgnoredState(state))
+    if (isIgnoredState(state, NULL))
       numstates++;
   }
   return numstates;
 }
 
-int PortList::numIgnoredPorts() {
+int PortList::numIgnoredPorts() const {
 
   int numports = 0;
+  int tmp = 0;
   for(int state=0; state < PORT_HIGHEST_STATE; state++) {
-    if (isIgnoredState(state))
-      numports += getStateCounts(state);
+    if (isIgnoredState(state, &tmp))
+      numports += tmp;
   }
   return numports;
 }
@@ -882,17 +859,52 @@ bool PortList::hasOpenPorts() const {
     getStateCounts(PORT_UNFILTERED) != 0;
 }
 
-int PortList::setStateReason(u16 portno, u8 proto, reason_t reason, u8 ttl, u32 ip_addr) {
+/* Returns true if service scan is done and portno is found to be tcpwrapped, false otherwise */
+bool PortList::isTCPwrapped(u16 portno) const {
+  const Port *port = lookupPort(portno, IPPROTO_TCP);
+  if (port == NULL) {
+    if (o.debugging > 1) {
+      log_write(LOG_STDOUT, "PortList::isTCPwrapped(%d) requested but port not in list\n", portno);
+    }
+    return false;
+  } else if (!o.servicescan) {
+    if (o.debugging > 1) {
+      log_write(LOG_STDOUT, "PortList::isTCPwrapped(%d) requested but service scan was never asked to be done\n", portno);
+    }
+    return false;
+  } else if (port->service == NULL) {
+    if (o.debugging > 1) {
+      log_write(LOG_STDOUT, "PortList::isTCPwrapped(%d) requested but port has not been service scanned yet\n", portno);
+    }
+    return false;
+  } else if (port->service->name == NULL) {
+    // no service match and port not listed in services file
+    if (o.debugging > 1) {
+      log_write(LOG_STDOUT, "PortList::isTCPwrapped(%d) requested but service has no name\n", portno);
+    }
+    return false;
+  } else {
+    return (strcmp(port->service->name,"tcpwrapped")==0);
+  }
+}
+
+int PortList::setStateReason(u16 portno, u8 proto, reason_t reason, u8 ttl,
+  const struct sockaddr_storage *ip_addr) {
     Port *answer = NULL;
 
-    if(reason > ER_MAX)
-        return -1;
     answer = createPort(portno, proto);
+    return setStateReason(answer, reason, ttl, ip_addr);
+}
 
+int PortList::setStateReason(Port *answer, reason_t reason, u8 ttl,
+  const struct sockaddr_storage *ip_addr) {
     /* set new reason and increment its count */
     answer->reason.reason_id = reason;
-    answer->reason.ip_addr.s_addr = ip_addr;
-	answer->reason.ttl = ttl;
+    if (ip_addr == NULL)
+      answer->reason.ip_addr.sockaddr.sa_family = AF_UNSPEC;
+    else
+      answer->reason.set_ip_addr(ip_addr);
+    answer->reason.ttl = ttl;
     return 0;
 }
 
@@ -918,15 +930,42 @@ void random_port_cheat(u16 *ports, int portcount) {
     // see if the currentport is a popular port
     for(popportidx = 0; popportidx < num_pop_ports; popportidx++) {
       if (ports[allportidx] == pop_ports[popportidx]) {
-	// This one is popular!  Swap it near to the beginning.
-	if (allportidx != earlyreplidx) {
-	  ports[allportidx] = ports[earlyreplidx];
-	  ports[earlyreplidx] = pop_ports[popportidx];
-	}
-	earlyreplidx++;
-	break;
+        // This one is popular!  Swap it near to the beginning.
+        if (allportidx != earlyreplidx) {
+          ports[allportidx] = ports[earlyreplidx];
+          ports[earlyreplidx] = pop_ports[popportidx];
+        }
+        earlyreplidx++;
+        break;
       }
     }
   }
 }
 
+const char *statenum2str(int state) {
+  switch (state) {
+  case PORT_OPEN:
+    return "open";
+    break;
+  case PORT_FILTERED:
+    return "filtered";
+    break;
+  case PORT_UNFILTERED:
+    return "unfiltered";
+    break;
+  case PORT_CLOSED:
+    return "closed";
+    break;
+  case PORT_OPENFILTERED:
+    return "open|filtered";
+    break;
+  case PORT_CLOSEDFILTERED:
+    return "closed|filtered";
+    break;
+  default:
+    return "unknown";
+    break;
+  }
+  return "unknown";
+}
+
