@@ -1,61 +1,59 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
+#!/usr/bin/env python3
 
 # ***********************IMPORTANT NMAP LICENSE TERMS************************
-# *                                                                         *
-# * The Nmap Security Scanner is (C) 1996-2022 Nmap Software LLC ("The Nmap *
-# * Project"). Nmap is also a registered trademark of the Nmap Project.     *
-# *                                                                         *
-# * This program is distributed under the terms of the Nmap Public Source   *
-# * License (NPSL). The exact license text applying to a particular Nmap    *
-# * release or source code control revision is contained in the LICENSE     *
-# * file distributed with that version of Nmap or source code control       *
-# * revision. More Nmap copyright/legal information is available from       *
-# * https://nmap.org/book/man-legal.html, and further information on the    *
-# * NPSL license itself can be found at https://nmap.org/npsl/ . This       *
-# * header summarizes some key points from the Nmap license, but is no      *
-# * substitute for the actual license text.                                 *
-# *                                                                         *
-# * Nmap is generally free for end users to download and use themselves,    *
-# * including commercial use. It is available from https://nmap.org.        *
-# *                                                                         *
-# * The Nmap license generally prohibits companies from using and           *
-# * redistributing Nmap in commercial products, but we sell a special Nmap  *
-# * OEM Edition with a more permissive license and special features for     *
-# * this purpose. See https://nmap.org/oem/                                 *
-# *                                                                         *
-# * If you have received a written Nmap license agreement or contract       *
-# * stating terms other than these (such as an Nmap OEM license), you may   *
-# * choose to use and redistribute Nmap under those terms instead.          *
-# *                                                                         *
-# * The official Nmap Windows builds include the Npcap software             *
-# * (https://npcap.com) for packet capture and transmission. It is under    *
-# * separate license terms which forbid redistribution without special      *
-# * permission. So the official Nmap Windows builds may not be              *
-# * redistributed without special permission (such as an Nmap OEM           *
-# * license).                                                               *
-# *                                                                         *
-# * Source is provided to this software because we believe users have a     *
-# * right to know exactly what a program is going to do before they run it. *
-# * This also allows you to audit the software for security holes.          *
-# *                                                                         *
-# * Source code also allows you to port Nmap to new platforms, fix bugs,    *
-# * and add new features.  You are highly encouraged to submit your         *
-# * changes as a Github PR or by email to the dev@nmap.org mailing list     *
-# * for possible incorporation into the main distribution. Unless you       *
-# * specify otherwise, it is understood that you are offering us very       *
-# * broad rights to use your submissions as described in the Nmap Public    *
-# * Source License Contributor Agreement. This is important because we      *
-# * fund the project by selling licenses with various terms, and also       *
-# * because the inability to relicense code has caused devastating          *
-# * problems for other Free Software projects (such as KDE and NASM).       *
-# *                                                                         *
-# * The free version of Nmap is distributed in the hope that it will be     *
-# * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of  *
-# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,        *
-# * indemnification and commercial support are all available through the    *
-# * Npcap OEM program--see https://nmap.org/oem/                            *
-# *                                                                         *
+# *
+# * The Nmap Security Scanner is (C) 1996-2024 Nmap Software LLC ("The Nmap
+# * Project"). Nmap is also a registered trademark of the Nmap Project.
+# *
+# * This program is distributed under the terms of the Nmap Public Source
+# * License (NPSL). The exact license text applying to a particular Nmap
+# * release or source code control revision is contained in the LICENSE
+# * file distributed with that version of Nmap or source code control
+# * revision. More Nmap copyright/legal information is available from
+# * https://nmap.org/book/man-legal.html, and further information on the
+# * NPSL license itself can be found at https://nmap.org/npsl/ . This
+# * header summarizes some key points from the Nmap license, but is no
+# * substitute for the actual license text.
+# *
+# * Nmap is generally free for end users to download and use themselves,
+# * including commercial use. It is available from https://nmap.org.
+# *
+# * The Nmap license generally prohibits companies from using and
+# * redistributing Nmap in commercial products, but we sell a special Nmap
+# * OEM Edition with a more permissive license and special features for
+# * this purpose. See https://nmap.org/oem/
+# *
+# * If you have received a written Nmap license agreement or contract
+# * stating terms other than these (such as an Nmap OEM license), you may
+# * choose to use and redistribute Nmap under those terms instead.
+# *
+# * The official Nmap Windows builds include the Npcap software
+# * (https://npcap.com) for packet capture and transmission. It is under
+# * separate license terms which forbid redistribution without special
+# * permission. So the official Nmap Windows builds may not be redistributed
+# * without special permission (such as an Nmap OEM license).
+# *
+# * Source is provided to this software because we believe users have a
+# * right to know exactly what a program is going to do before they run it.
+# * This also allows you to audit the software for security holes.
+# *
+# * Source code also allows you to port Nmap to new platforms, fix bugs, and
+# * add new features. You are highly encouraged to submit your changes as a
+# * Github PR or by email to the dev@nmap.org mailing list for possible
+# * incorporation into the main distribution. Unless you specify otherwise, it
+# * is understood that you are offering us very broad rights to use your
+# * submissions as described in the Nmap Public Source License Contributor
+# * Agreement. This is important because we fund the project by selling licenses
+# * with various terms, and also because the inability to relicense code has
+# * caused devastating problems for other Free Software projects (such as KDE
+# * and NASM).
+# *
+# * The free version of Nmap is distributed in the hope that it will be
+# * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,
+# * indemnification and commercial support are all available through the
+# * Npcap OEM program--see https://nmap.org/oem/
+# *
 # ***************************************************************************/
 
 import locale
@@ -63,11 +61,7 @@ import time
 import socket
 import copy
 
-# Use the faster cStringIO if available, fallback on StringIO if not
-try:
-    from cStringIO import StringIO
-except ImportError:
-    from StringIO import StringIO
+from io import StringIO
 
 # Prevent loading PyXML
 import xml
@@ -260,7 +254,7 @@ class HostInfo(object):
         1) IPv4 comes before IPv6 comes before MAC, and
         2) addresses are sorted according to their binary values, not their
            string representation.
-        Use this function to the the comparison key when sorting a list of
+        Use this function to the comparison key when sorting a list of
         hosts by address."""
         l = []
         if self.ip:
@@ -359,7 +353,7 @@ class HostInfo(object):
                     _("Unknown version")),
                 "service_product": p.get("service_product", ""),
                 "service_extrainfo": p.get("service_extrainfo", ""),
-                "port_state": p.get("port_state", _("Unknown")),
+                "port_state": p.get("port_state", _("unknown")),
                 "protocol": p.get("protocol", "")
                 })
         return services
@@ -558,12 +552,7 @@ in epoch format!")
         return ports
 
     def get_formatted_date(self):
-        try:
-            return time.strftime("%B %d, %Y - %H:%M", self.get_date()).decode(
-                locale.getpreferredencoding())
-        except LookupError:
-            # encoding or locale not found
-            return time.asctime(self.get_date()).decode('ascii')
+        return time.strftime("%B %d, %Y - %H:%M", self.get_date())
 
     def get_scanner(self):
         return self.nmap['nmaprun'].get('scanner', '')
@@ -1044,7 +1033,7 @@ class NmapParserSAX(ParserBasics, ContentHandler):
     def write_xml_to_file(self, filename):
         """Write the XML representation of this scan to the file whose name is
         given."""
-        fd = open(filename, "wb")
+        fd = open(filename, "w")
         self.write_xml(fd)
         fd.close()
 
@@ -1333,25 +1322,25 @@ if __name__ == '__main__':
     np.parse_file(file_to_parse)
 
     for host in np.hosts:
-        print "%s:" % host.ip["addr"]
-        print "  Comment:", repr(host.comment)
-        print "  TCP sequence:", repr(host.tcpsequence)
-        print "  TCP TS sequence:", repr(host.tcptssequence)
-        print "  IP ID sequence:", repr(host.ipidsequence)
-        print "  Uptime:", repr(host.uptime)
-        print "  OS Match:", repr(host.osmatches)
-        print "  Ports:"
+        print("%s:" % host.ip["addr"])
+        print("  Comment:", repr(host.comment))
+        print("  TCP sequence:", repr(host.tcpsequence))
+        print("  TCP TS sequence:", repr(host.tcptssequence))
+        print("  IP ID sequence:", repr(host.ipidsequence))
+        print("  Uptime:", repr(host.uptime))
+        print("  OS Match:", repr(host.osmatches))
+        print("  Ports:")
         for p in host.ports:
-            print "\t%s" % repr(p)
-        print "  Ports used:", repr(host.ports_used)
-        print "  OS Matches:", repr(host.osmatches)
-        print "  Hostnames:", repr(host.hostnames)
-        print "  IP:", repr(host.ip)
-        print "  IPv6:", repr(host.ipv6)
-        print "  MAC:", repr(host.mac)
-        print "  State:", repr(host.state)
+            print("\t%s" % repr(p))
+        print("  Ports used:", repr(host.ports_used))
+        print("  OS Matches:", repr(host.osmatches))
+        print("  Hostnames:", repr(host.hostnames))
+        print("  IP:", repr(host.ip))
+        print("  IPv6:", repr(host.ipv6))
+        print("  MAC:", repr(host.mac))
+        print("  State:", repr(host.state))
         if "hops" in host.trace:
-            print "  Trace:"
+            print("  Trace:")
             for hop in host.trace["hops"]:
-                print "    ", repr(hop)
-            print
+                print("    ", repr(hop))
+            print()
