@@ -1,68 +1,65 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
+#!/usr/bin/env python3
 
 # ***********************IMPORTANT NMAP LICENSE TERMS************************
-# *                                                                         *
-# * The Nmap Security Scanner is (C) 1996-2022 Nmap Software LLC ("The Nmap *
-# * Project"). Nmap is also a registered trademark of the Nmap Project.     *
-# *                                                                         *
-# * This program is distributed under the terms of the Nmap Public Source   *
-# * License (NPSL). The exact license text applying to a particular Nmap    *
-# * release or source code control revision is contained in the LICENSE     *
-# * file distributed with that version of Nmap or source code control       *
-# * revision. More Nmap copyright/legal information is available from       *
-# * https://nmap.org/book/man-legal.html, and further information on the    *
-# * NPSL license itself can be found at https://nmap.org/npsl/ . This       *
-# * header summarizes some key points from the Nmap license, but is no      *
-# * substitute for the actual license text.                                 *
-# *                                                                         *
-# * Nmap is generally free for end users to download and use themselves,    *
-# * including commercial use. It is available from https://nmap.org.        *
-# *                                                                         *
-# * The Nmap license generally prohibits companies from using and           *
-# * redistributing Nmap in commercial products, but we sell a special Nmap  *
-# * OEM Edition with a more permissive license and special features for     *
-# * this purpose. See https://nmap.org/oem/                                 *
-# *                                                                         *
-# * If you have received a written Nmap license agreement or contract       *
-# * stating terms other than these (such as an Nmap OEM license), you may   *
-# * choose to use and redistribute Nmap under those terms instead.          *
-# *                                                                         *
-# * The official Nmap Windows builds include the Npcap software             *
-# * (https://npcap.com) for packet capture and transmission. It is under    *
-# * separate license terms which forbid redistribution without special      *
-# * permission. So the official Nmap Windows builds may not be              *
-# * redistributed without special permission (such as an Nmap OEM           *
-# * license).                                                               *
-# *                                                                         *
-# * Source is provided to this software because we believe users have a     *
-# * right to know exactly what a program is going to do before they run it. *
-# * This also allows you to audit the software for security holes.          *
-# *                                                                         *
-# * Source code also allows you to port Nmap to new platforms, fix bugs,    *
-# * and add new features.  You are highly encouraged to submit your         *
-# * changes as a Github PR or by email to the dev@nmap.org mailing list     *
-# * for possible incorporation into the main distribution. Unless you       *
-# * specify otherwise, it is understood that you are offering us very       *
-# * broad rights to use your submissions as described in the Nmap Public    *
-# * Source License Contributor Agreement. This is important because we      *
-# * fund the project by selling licenses with various terms, and also       *
-# * because the inability to relicense code has caused devastating          *
-# * problems for other Free Software projects (such as KDE and NASM).       *
-# *                                                                         *
-# * The free version of Nmap is distributed in the hope that it will be     *
-# * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of  *
-# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,        *
-# * indemnification and commercial support are all available through the    *
-# * Npcap OEM program--see https://nmap.org/oem/                            *
-# *                                                                         *
+# *
+# * The Nmap Security Scanner is (C) 1996-2024 Nmap Software LLC ("The Nmap
+# * Project"). Nmap is also a registered trademark of the Nmap Project.
+# *
+# * This program is distributed under the terms of the Nmap Public Source
+# * License (NPSL). The exact license text applying to a particular Nmap
+# * release or source code control revision is contained in the LICENSE
+# * file distributed with that version of Nmap or source code control
+# * revision. More Nmap copyright/legal information is available from
+# * https://nmap.org/book/man-legal.html, and further information on the
+# * NPSL license itself can be found at https://nmap.org/npsl/ . This
+# * header summarizes some key points from the Nmap license, but is no
+# * substitute for the actual license text.
+# *
+# * Nmap is generally free for end users to download and use themselves,
+# * including commercial use. It is available from https://nmap.org.
+# *
+# * The Nmap license generally prohibits companies from using and
+# * redistributing Nmap in commercial products, but we sell a special Nmap
+# * OEM Edition with a more permissive license and special features for
+# * this purpose. See https://nmap.org/oem/
+# *
+# * If you have received a written Nmap license agreement or contract
+# * stating terms other than these (such as an Nmap OEM license), you may
+# * choose to use and redistribute Nmap under those terms instead.
+# *
+# * The official Nmap Windows builds include the Npcap software
+# * (https://npcap.com) for packet capture and transmission. It is under
+# * separate license terms which forbid redistribution without special
+# * permission. So the official Nmap Windows builds may not be redistributed
+# * without special permission (such as an Nmap OEM license).
+# *
+# * Source is provided to this software because we believe users have a
+# * right to know exactly what a program is going to do before they run it.
+# * This also allows you to audit the software for security holes.
+# *
+# * Source code also allows you to port Nmap to new platforms, fix bugs, and
+# * add new features. You are highly encouraged to submit your changes as a
+# * Github PR or by email to the dev@nmap.org mailing list for possible
+# * incorporation into the main distribution. Unless you specify otherwise, it
+# * is understood that you are offering us very broad rights to use your
+# * submissions as described in the Nmap Public Source License Contributor
+# * Agreement. This is important because we fund the project by selling licenses
+# * with various terms, and also because the inability to relicense code has
+# * caused devastating problems for other Free Software projects (such as KDE
+# * and NASM).
+# *
+# * The free version of Nmap is distributed in the hope that it will be
+# * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,
+# * indemnification and commercial support are all available through the
+# * Npcap OEM program--see https://nmap.org/oem/
+# *
 # ***************************************************************************/
 
 import re
 
-from types import StringTypes
-from ConfigParser import DuplicateSectionError, NoSectionError, NoOptionError
-from ConfigParser import Error as ConfigParser_Error
+from configparser import DuplicateSectionError, NoSectionError, NoOptionError
+from configparser import Error as ConfigParser_Error
 
 from zenmapCore.Paths import Path
 from zenmapCore.UmitLogging import log
@@ -107,7 +104,7 @@ class SearchConfig(UmitConfigParser, object):
         self.search_db = True
 
     def _get_it(self, p_name, default):
-        return config_parser.get(self.section_name, p_name, default)
+        return config_parser.get(self.section_name, p_name, fallback=default)
 
     def _set_it(self, p_name, value):
         config_parser.set(self.section_name, p_name, value)
@@ -117,10 +114,8 @@ class SearchConfig(UmitConfigParser, object):
            attr == "True" or \
            attr == "true" or \
            attr == "1":
-
-            return 1
-
-        return 0
+            return "True"
+        return "False"
 
     def get_directory(self):
         return self._get_it("directory", "")
@@ -134,7 +129,7 @@ class SearchConfig(UmitConfigParser, object):
     def set_file_extension(self, file_extension):
         if isinstance(file_extension, list):
             self._set_it("file_extension", ";".join(file_extension))
-        elif isinstance(file_extension, StringTypes):
+        elif isinstance(file_extension, str):
             self._set_it("file_extension", file_extension)
 
     def get_save_time(self):
@@ -143,7 +138,7 @@ class SearchConfig(UmitConfigParser, object):
     def set_save_time(self, save_time):
         if isinstance(save_time, list):
             self._set_it("save_time", ";".join(save_time))
-        elif isinstance(save_time, StringTypes):
+        elif isinstance(save_time, str):
             self._set_it("save_time", save_time)
 
     def get_store_results(self):
@@ -272,7 +267,7 @@ class WindowConfig(UmitConfigParser, object):
         self.height = self.default_height
 
     def _get_it(self, p_name, default):
-        return config_parser.get(self.section_name, p_name, default)
+        return config_parser.get(self.section_name, p_name, fallback=default)
 
     def _set_it(self, p_name, value):
         config_parser.set(self.section_name, p_name, value)
@@ -401,7 +396,7 @@ class NmapOutputHighlight(object):
         try:
             return self.sanity_settings([
                 config_parser.get(
-                    property_name, prop, True) for prop in self.setts])
+                    property_name, prop, raw=True) for prop in self.setts])
         except Exception:
             settings = []
             prop_settings = self.default_highlights[p_name]
@@ -420,7 +415,7 @@ class NmapOutputHighlight(object):
         property_name = "%s_highlight" % property_name
         settings = self.sanity_settings(list(settings))
 
-        for pos in xrange(len(settings)):
+        for pos in range(len(settings)):
             config_parser.set(property_name, self.setts[pos], settings[pos])
 
     def sanity_settings(self, settings):
@@ -437,13 +432,13 @@ class NmapOutputHighlight(object):
         settings[1] = self.boolean_sanity(settings[1])
         settings[2] = self.boolean_sanity(settings[2])
 
-        tuple_regex = "[\(\[]\s?(\d+)\s?,\s?(\d+)\s?,\s?(\d+)\s?[\)\]]"
-        if isinstance(settings[3], basestring):
+        tuple_regex = r"[\(\[]\s?(\d+)\s?,\s?(\d+)\s?,\s?(\d+)\s?[\)\]]"
+        if isinstance(settings[3], str):
             settings[3] = [
                     int(t) for t in re.findall(tuple_regex, settings[3])[0]
                     ]
 
-        if isinstance(settings[4], basestring):
+        if isinstance(settings[4], str):
             settings[4] = [
                     int(h) for h in re.findall(tuple_regex, settings[4])[0]
                     ]
@@ -542,57 +537,57 @@ class NmapOutputHighlight(object):
                 "underline": str(False),
                 "text": [0, 0, 0],
                 "highlight": [65535, 65535, 65535],
-                "regex": "\d{4}-\d{2}-\d{2}\s\d{2}:\d{2}\s.{1,4}"},
+                "regex": r"\d{4}-\d{2}-\d{2}\s\d{2}:\d{2}\s.{1,4}"},
             "hostname": {
                 "bold": str(True),
                 "italic": str(True),
                 "underline": str(True),
                 "text": [0, 111, 65535],
                 "highlight": [65535, 65535, 65535],
-                "regex": "(\w{2,}://)*[\w-]{2,}\.[\w-]{2,}"
-                         "(\.[\w-]{2,})*(/[[\w-]{2,}]*)*"},
+                "regex": r"(\w{2,}://)*[\w-]{2,}\.[\w-]{2,}"
+                         r"(\.[\w-]{2,})*(/[[\w-]{2,}]*)*"},
             "ip": {
                 "bold": str(True),
                 "italic": str(False),
                 "underline": str(False),
                 "text": [0, 0, 0],
                 "highlight": [65535, 65535, 65535],
-                "regex": "\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}"},
+                "regex": r"\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}"},
             "port_list": {
                 "bold": str(True),
                 "italic": str(False),
                 "underline": str(False),
                 "text": [0, 1272, 28362],
                 "highlight": [65535, 65535, 65535],
-                "regex": "PORT\s+STATE\s+SERVICE(\s+VERSION)?[^\n]*"},
+                "regex": r"PORT\s+STATE\s+SERVICE(\s+VERSION)?[^\n]*"},
             "open_port": {
                 "bold": str(True),
                 "italic": str(False),
                 "underline": str(False),
                 "text": [0, 41036, 2396],
                 "highlight": [65535, 65535, 65535],
-                "regex": "\d{1,5}/.{1,5}\s+open\s+.*"},
+                "regex": r"\d{1,5}/.{1,5}\s+open\s+.*"},
             "closed_port": {
                 "bold": str(False),
                 "italic": str(False),
                 "underline": str(False),
                 "text": [65535, 0, 0],
                 "highlight": [65535, 65535, 65535],
-                "regex": "\d{1,5}/.{1,5}\s+closed\s+.*"},
+                "regex": r"\d{1,5}/.{1,5}\s+closed\s+.*"},
             "filtered_port": {
                 "bold": str(False),
                 "italic": str(False),
                 "underline": str(False),
                 "text": [38502, 39119, 0],
                 "highlight": [65535, 65535, 65535],
-                "regex": "\d{1,5}/.{1,5}\s+filtered\s+.*"},
+                "regex": r"\d{1,5}/.{1,5}\s+filtered\s+.*"},
             "details": {
                 "bold": str(True),
                 "italic": str(False),
                 "underline": str(True),
                 "text": [0, 0, 0],
                 "highlight": [65535, 65535, 65535],
-                "regex": "^(\w{2,}[\s]{,3}){,4}:"}
+                "regex": r"^(\w{2,}[\s]{,3}){,4}:"}
             }
 
 
