@@ -1,64 +1,65 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
+#!/usr/bin/env python3
 
 # ***********************IMPORTANT NMAP LICENSE TERMS************************
-# *                                                                         *
-# * The Nmap Security Scanner is (C) 1996-2022 Nmap Software LLC ("The Nmap *
-# * Project"). Nmap is also a registered trademark of the Nmap Project.     *
-# *                                                                         *
-# * This program is distributed under the terms of the Nmap Public Source   *
-# * License (NPSL). The exact license text applying to a particular Nmap    *
-# * release or source code control revision is contained in the LICENSE     *
-# * file distributed with that version of Nmap or source code control       *
-# * revision. More Nmap copyright/legal information is available from       *
-# * https://nmap.org/book/man-legal.html, and further information on the    *
-# * NPSL license itself can be found at https://nmap.org/npsl/ . This       *
-# * header summarizes some key points from the Nmap license, but is no      *
-# * substitute for the actual license text.                                 *
-# *                                                                         *
-# * Nmap is generally free for end users to download and use themselves,    *
-# * including commercial use. It is available from https://nmap.org.        *
-# *                                                                         *
-# * The Nmap license generally prohibits companies from using and           *
-# * redistributing Nmap in commercial products, but we sell a special Nmap  *
-# * OEM Edition with a more permissive license and special features for     *
-# * this purpose. See https://nmap.org/oem/                                 *
-# *                                                                         *
-# * If you have received a written Nmap license agreement or contract       *
-# * stating terms other than these (such as an Nmap OEM license), you may   *
-# * choose to use and redistribute Nmap under those terms instead.          *
-# *                                                                         *
-# * The official Nmap Windows builds include the Npcap software             *
-# * (https://npcap.com) for packet capture and transmission. It is under    *
-# * separate license terms which forbid redistribution without special      *
-# * permission. So the official Nmap Windows builds may not be              *
-# * redistributed without special permission (such as an Nmap OEM           *
-# * license).                                                               *
-# *                                                                         *
-# * Source is provided to this software because we believe users have a     *
-# * right to know exactly what a program is going to do before they run it. *
-# * This also allows you to audit the software for security holes.          *
-# *                                                                         *
-# * Source code also allows you to port Nmap to new platforms, fix bugs,    *
-# * and add new features.  You are highly encouraged to submit your         *
-# * changes as a Github PR or by email to the dev@nmap.org mailing list     *
-# * for possible incorporation into the main distribution. Unless you       *
-# * specify otherwise, it is understood that you are offering us very       *
-# * broad rights to use your submissions as described in the Nmap Public    *
-# * Source License Contributor Agreement. This is important because we      *
-# * fund the project by selling licenses with various terms, and also       *
-# * because the inability to relicense code has caused devastating          *
-# * problems for other Free Software projects (such as KDE and NASM).       *
-# *                                                                         *
-# * The free version of Nmap is distributed in the hope that it will be     *
-# * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of  *
-# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,        *
-# * indemnification and commercial support are all available through the    *
-# * Npcap OEM program--see https://nmap.org/oem/                            *
-# *                                                                         *
+# *
+# * The Nmap Security Scanner is (C) 1996-2024 Nmap Software LLC ("The Nmap
+# * Project"). Nmap is also a registered trademark of the Nmap Project.
+# *
+# * This program is distributed under the terms of the Nmap Public Source
+# * License (NPSL). The exact license text applying to a particular Nmap
+# * release or source code control revision is contained in the LICENSE
+# * file distributed with that version of Nmap or source code control
+# * revision. More Nmap copyright/legal information is available from
+# * https://nmap.org/book/man-legal.html, and further information on the
+# * NPSL license itself can be found at https://nmap.org/npsl/ . This
+# * header summarizes some key points from the Nmap license, but is no
+# * substitute for the actual license text.
+# *
+# * Nmap is generally free for end users to download and use themselves,
+# * including commercial use. It is available from https://nmap.org.
+# *
+# * The Nmap license generally prohibits companies from using and
+# * redistributing Nmap in commercial products, but we sell a special Nmap
+# * OEM Edition with a more permissive license and special features for
+# * this purpose. See https://nmap.org/oem/
+# *
+# * If you have received a written Nmap license agreement or contract
+# * stating terms other than these (such as an Nmap OEM license), you may
+# * choose to use and redistribute Nmap under those terms instead.
+# *
+# * The official Nmap Windows builds include the Npcap software
+# * (https://npcap.com) for packet capture and transmission. It is under
+# * separate license terms which forbid redistribution without special
+# * permission. So the official Nmap Windows builds may not be redistributed
+# * without special permission (such as an Nmap OEM license).
+# *
+# * Source is provided to this software because we believe users have a
+# * right to know exactly what a program is going to do before they run it.
+# * This also allows you to audit the software for security holes.
+# *
+# * Source code also allows you to port Nmap to new platforms, fix bugs, and
+# * add new features. You are highly encouraged to submit your changes as a
+# * Github PR or by email to the dev@nmap.org mailing list for possible
+# * incorporation into the main distribution. Unless you specify otherwise, it
+# * is understood that you are offering us very broad rights to use your
+# * submissions as described in the Nmap Public Source License Contributor
+# * Agreement. This is important because we fund the project by selling licenses
+# * with various terms, and also because the inability to relicense code has
+# * caused devastating problems for other Free Software projects (such as KDE
+# * and NASM).
+# *
+# * The free version of Nmap is distributed in the hope that it will be
+# * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,
+# * indemnification and commercial support are all available through the
+# * Npcap OEM program--see https://nmap.org/oem/
+# *
 # ***************************************************************************/
 
-import gtk
+import gi
+
+gi.require_version("Gtk", "3.0")
+from gi.repository import Gtk
 
 from zenmapGUI.higwidgets.higexpanders import HIGExpander
 from zenmapGUI.higwidgets.higboxes import HIGVBox, HIGHBox,\
@@ -121,21 +122,21 @@ class HostDetails(HIGVBox):
         self.set_comment(host.comment)
 
     def __create_widgets(self):
-        self.host_status_expander = gtk.Expander(
+        self.host_status_expander = Gtk.Expander.new(
                 '<b>' + _('Host Status') + '</b>')
-        self.address_expander = gtk.Expander('<b>' + _('Addresses') + '</b>')
-        self.hostnames_expander = gtk.Expander('<b>' + _('Hostnames') + '</b>')
-        self.os_expander = gtk.Expander('<b>' + _('Operating System') + '</b>')
-        self.portsused_expander = gtk.Expander(
+        self.address_expander = Gtk.Expander.new('<b>' + _('Addresses') + '</b>')
+        self.hostnames_expander = Gtk.Expander.new('<b>' + _('Hostnames') + '</b>')
+        self.os_expander = Gtk.Expander.new('<b>' + _('Operating System') + '</b>')
+        self.portsused_expander = Gtk.Expander.new(
                 '<b>' + _('Ports used') + '</b>')
-        self.osclass_expander = gtk.Expander('<b>' + _('OS Classes') + '</b>')
-        self.tcp_expander = gtk.Expander('<b>' + _('TCP Sequence') + '</b>')
-        self.ip_expander = gtk.Expander('<b>' + _('IP ID Sequence') + '</b>')
-        self.tcpts_expander = gtk.Expander(
+        self.osclass_expander = Gtk.Expander.new('<b>' + _('OS Classes') + '</b>')
+        self.tcp_expander = Gtk.Expander.new('<b>' + _('TCP Sequence') + '</b>')
+        self.ip_expander = Gtk.Expander.new('<b>' + _('IP ID Sequence') + '</b>')
+        self.tcpts_expander = Gtk.Expander.new(
                 '<b>' + _('TCP TS Sequence') + '</b>')
-        self.comment_expander = gtk.Expander('<b>' + _('Comments') + '</b>')
-        self.os_image = gtk.Image()
-        self.vulnerability_image = gtk.Image()
+        self.comment_expander = Gtk.Expander.new('<b>' + _('Comments') + '</b>')
+        self.os_image = Gtk.Image()
+        self.vulnerability_image = Gtk.Image()
 
         # Host Status expander
         self.host_state_label = HIGEntryLabel(_('State:'))
@@ -160,13 +161,13 @@ class HostDetails(HIGVBox):
         self.info_lastboot_label = HIGEntryLabel(na)
 
         # Addresses expander
-        self.ipv4_label = HIGEntryLabel(_('IPv4:'))
+        self.ipv4_label = HIGEntryLabel('IPv4:')
         self.info_ipv4_label = HIGEntryLabel(na)
 
-        self.ipv6_label = HIGEntryLabel(_('IPv6:'))
+        self.ipv6_label = HIGEntryLabel('IPv6:')
         self.info_ipv6_label = HIGEntryLabel(na)
 
-        self.mac_label = HIGEntryLabel(_('MAC:'))
+        self.mac_label = HIGEntryLabel('MAC:')
         self.info_mac_label = HIGEntryLabel(na)
 
         self.vendor_label = HIGEntryLabel(_('Vendor:'))
@@ -235,9 +236,9 @@ class HostDetails(HIGVBox):
         table.attach(self.lastboot_label, 0, 1, 6, 7)
         table.attach(self.info_lastboot_label, 1, 2, 6, 7)
 
-        table.attach(self.os_image, 2, 4, 0, 3, xoptions=1, yoptions=0)
+        table.attach(self.os_image, 2, 4, 0, 3, xoptions=Gtk.AttachOptions.EXPAND)
         table.attach(
-                self.vulnerability_image, 2, 4, 4, 7, xoptions=1, yoptions=0)
+                self.vulnerability_image, 2, 4, 4, 7, xoptions=Gtk.AttachOptions.EXPAND)
 
         table.set_col_spacing(1, 50)
 
@@ -245,10 +246,10 @@ class HostDetails(HIGVBox):
         self._pack_noexpand_nofill(self.host_status_expander)
 
     def set_os_image(self, image):
-            self.os_image.set_from_stock(image, gtk.ICON_SIZE_DIALOG)
+            self.os_image.set_from_stock(image, Gtk.IconSize.DIALOG)
 
     def set_vulnerability_image(self, image):
-        self.vulnerability_image.set_from_stock(image, gtk.ICON_SIZE_DIALOG)
+        self.vulnerability_image.set_from_stock(image, Gtk.IconSize.DIALOG)
 
     def set_addresses(self, address):
         self.address_expander.set_use_markup(True)
@@ -306,7 +307,7 @@ class HostDetails(HIGVBox):
             self.os_expander.set_use_markup(True)
             self.os_expander.set_expanded(True)
             table, hbox = self.create_table_hbox()
-            progress = gtk.ProgressBar()
+            progress = Gtk.ProgressBar()
 
             if 'accuracy' in os:
                 progress.set_fraction(float(os['accuracy']) / 100.0)
@@ -375,7 +376,7 @@ class HostDetails(HIGVBox):
                 table.attach(HIGEntryLabel(o['osfamily']), 2, 3, y1, y2)
                 table.attach(HIGEntryLabel(o['osgen']), 3, 4, y1, y2)
 
-                progress = gtk.ProgressBar()
+                progress = Gtk.ProgressBar()
                 progress.set_text(o['accuracy'] + '%')
                 progress.set_fraction(float(o['accuracy']) / 100.0)
                 table.attach(progress, 4, 5, y1, y2)
@@ -389,7 +390,7 @@ class HostDetails(HIGVBox):
             self.tcp_expander.set_use_markup(True)
             table, hbox = self.create_table_hbox()
 
-            combo = gtk.combo_box_new_text()
+            combo = Gtk.ComboBoxText()
             for v in tcpseq['values'].split(','):
                 combo.append_text(v)
 
@@ -410,7 +411,7 @@ class HostDetails(HIGVBox):
             self.ip_expander.set_use_markup(True)
             table, hbox = self.create_table_hbox()
 
-            combo = gtk.combo_box_new_text()
+            combo = Gtk.ComboBoxText()
 
             for i in ipseq['values'].split(','):
                 combo.append_text(i)
@@ -429,7 +430,7 @@ class HostDetails(HIGVBox):
             self.tcpts_expander.set_use_markup(True)
             table, hbox = self.create_table_hbox()
 
-            combo = gtk.combo_box_new_text()
+            combo = Gtk.ComboBoxText()
 
             for i in tcptsseq['values'].split(','):
                 combo.append_text(i)
@@ -450,13 +451,13 @@ class HostDetails(HIGVBox):
 
         hbox = HIGHBox()
 
-        self.comment_scrolled = gtk.ScrolledWindow()
+        self.comment_scrolled = Gtk.ScrolledWindow()
         self.comment_scrolled.set_border_width(5)
         self.comment_scrolled.set_policy(
-                gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+                Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
 
-        self.comment_txt_vw = gtk.TextView()
-        self.comment_txt_vw.set_wrap_mode(gtk.WRAP_WORD)
+        self.comment_txt_vw = Gtk.TextView()
+        self.comment_txt_vw.set_wrap_mode(Gtk.WrapMode.WORD)
         self.comment_txt_vw.get_buffer().set_text(comment)
 
         self.comment_scrolled.add(self.comment_txt_vw)
@@ -467,4 +468,5 @@ class HostDetails(HIGVBox):
 
     def get_comment(self):
         buffer = self.comment_txt_vw.get_buffer()
-        return buffer.get_text(buffer.get_start_iter(), buffer.get_end_iter())
+        return buffer.get_text(buffer.get_start_iter(), buffer.get_end_iter(),
+                include_hidden_chars=True)
